@@ -1,3 +1,4 @@
+/* $Id: CoinPresolveDupcol.cpp 1270 2010-04-06 00:09:31Z ladanyi $ */
 // Copyright (C) 2002, International Business Machines
 // Corporation and others.  All Rights Reserved.
 #include <stdio.h>
@@ -40,7 +41,7 @@ namespace {	// begin unnamed file-local namespace
   The sums are returned in the corresponding entries of majsums.
 */
 
-void compute_sums (int n, const int *majlens, const CoinBigIndex *majstrts,
+  void compute_sums (int /*n*/, const int *majlens, const CoinBigIndex *majstrts,
 		   int *minndxs, double *elems, const double *minmuls,
 		   int *majcands, double *majsums, int nlook)
 
@@ -701,7 +702,10 @@ const CoinPresolveAction
 	   nactions,nfixed_down,nfixed_up) ; }
 # endif
   if (nactions)
-  { next = new dupcol_action(nactions,CoinCopyOfArray(actions,nactions),next) ; }
+  { next = new dupcol_action(nactions,CoinCopyOfArray(actions,nactions),next) ;
+    // we can't go round again in integer
+    prob->presolveOptions_ |= 0x80000000;
+}
   deleteAction(actions,action*) ;
 
   if (nfixed_down)
@@ -892,6 +896,7 @@ const CoinPresolveAction
 
   int nuseless_rows = 0;
   bool fixInfeasibility = (prob->presolveOptions_&16384)!=0;
+  bool allowIntersection = ( prob->presolveOptions_&16)!=0;
   double tolerance = prob->feasibilityTolerance_;
 
   double dval = workrow[0];
@@ -929,11 +934,6 @@ const CoinPresolveAction
 	      sort[jj-1]=ithis;
 	      sort[jj]=ilast;
 	    } else {
-	      /* overlapping - could merge */
-#	      if PRESOLVE_DEBUG
-	      printf("overlapping duplicate row %g %g, %g %g\n",
-		     rlo1,rup1,rlo2,rup2);
-#	      endif
 	      if (rup1<rlo2-tolerance&&!fixInfeasibility) {
 		// infeasible
 		prob->status_|= 1;
@@ -945,10 +945,19 @@ const CoinPresolveAction
 						  <<rup[ithis]
 						  <<CoinMessageEol;
 		break;
+	      } else if (allowIntersection/*||fabs(rup1-rlo2)<tolerance*/) {
+		/* overlapping - could merge */
+#ifdef CLP_INVESTIGATE7
+		printf("overlapping duplicate row %g %g, %g %g\n",
+		       rlo1,rup1,rlo2,rup2);
+#	      endif
+		// pretend this is stricter than last
+		idelete=ilast;
+		rup[ithis]=rup1;
 	      }
 	    }
 	  } else {
-	    // rlo2<=rlo1
+	    // rlo1>rlo2
 	    if (rup1<=rup2) {
 	      /* last is strictly tighter than this */
 	      idelete=ithis;
@@ -957,10 +966,8 @@ const CoinPresolveAction
 	      sort[jj]=ilast;
 	    } else {
 	      /* overlapping - could merge */
-#	      if PRESOLVE_DEBUG
-	      printf("overlapping duplicate row %g %g, %g %g\n",
-		     rlo1,rup1,rlo2,rup2);
-#	      endif
+	      // rlo1>rlo2
+	      // rup1>rup2 
 	      if (rup2<rlo1-tolerance&&!fixInfeasibility) {
 		// infeasible
 		prob->status_|= 1;
@@ -972,6 +979,14 @@ const CoinPresolveAction
 						  <<rup[ithis]
 						  <<CoinMessageEol;
 		break;
+	      } else if (allowIntersection/*||fabs(rup2-rlo1)<tolerance*/) {
+#ifdef CLP_INVESTIGATE7
+		printf("overlapping duplicate row %g %g, %g %g\n",
+		       rlo1,rup1,rlo2,rup2);
+#	      endif
+		// pretend this is stricter than last
+		idelete=ilast;
+		rlo[ithis]=rlo1;
 	      }
 	    }
 	  }
@@ -1008,7 +1023,7 @@ const CoinPresolveAction
   return (next);
 }
 
-void duprow_action::postsolve(CoinPostsolveMatrix *prob) const
+void duprow_action::postsolve(CoinPostsolveMatrix *) const
 {
   printf("STILL NO POSTSOLVE FOR DUPROW!\n");
   abort();
@@ -1149,7 +1164,7 @@ const CoinPresolveAction
   return (next);
 }
 
-void gubrow_action::postsolve(CoinPostsolveMatrix *prob) const
+void gubrow_action::postsolve(CoinPostsolveMatrix *) const
 {
   printf("STILL NO POSTSOLVE FOR GUBROW!\n");
   abort();
