@@ -141,7 +141,7 @@ typedef struct { int minmax ;
 
 
 static char sccsid[] UNUSED = "@(#)mpsio.c	4.4	11/06/04" ;
-static char svnid[] UNUSED = "$Id: mpsio.c 210 2008-02-22 18:23:36Z lou $" ;
+static char svnid[] UNUSED = "$Id: mpsio.c 344 2010-07-29 20:13:26Z lou $" ;
 
 /*
   A bunch of definitions and declarations for mpsin and its slave routines.
@@ -190,8 +190,9 @@ static lex_struct *getmpsline (ioid mpschn)
   Parameters:
     mpschn:	stream for the MPS input
 
-  Returns: a pointer to a lex_struct; on normal return, the type will be LCQS;
-	   other possibilities are LCEOF (end-of-file) and LCERR (i/o error)
+  Returns: a pointer to a lex_struct; on normal return, the type will be
+	   DY_LCQS; other possibilities are DY_LCEOF (end-of-file) and
+	   DY_LCERR (i/o error)
 
   NOTE: The lex_struct returned by getmpsline is owned by dyio_scanstr. The
 	string buffer it contains is allocated by scanstr and will be freed
@@ -206,15 +207,15 @@ static lex_struct *getmpsline (ioid mpschn)
 
 /*
   Fire up a loop to read lines. A line that's all white space will come back
-  from dyio_scanstr as LCNIL. If there's something in the line, check for
+  from dyio_scanstr as DY_LCNIL. If there's something in the line, check for
   comments.  If there's still a line left after peeling off the comment,
   we've got something.
 */
   empty = TRUE ;
   while (empty == TRUE)
-  { lex = dyio_scanstr(mpschn,LCQS,0,'\0','\n') ;
-    if (lex->class == LCEOF || lex->class == LCERR) break ;
-    if (lex->class == LCNIL) continue ;
+  { lex = dyio_scanstr(mpschn,DY_LCQS,0,'\0','\n') ;
+    if (lex->class == DY_LCEOF || lex->class == DY_LCERR) break ;
+    if (lex->class == DY_LCNIL) continue ;
 /*
   If the last character is ^M (carriage return), strip it. Likely we're looking
   at a file produced on a system where <cr><lf> ends a line.
@@ -270,10 +271,10 @@ static mpsinstate_enum mpsin_name (ioid mpschn, consys_struct **consys,
   Find the "name" keyword.
 */
   lex = getmpsline(mpschn) ;
-  if (lex->class == LCERR)
+  if (lex->class == DY_LCERR)
   { errmsg(168,rtnnme,mysection) ;
     return (mpsinINV) ; }
-  if (lex->class == LCEOF)
+  if (lex->class == DY_LCEOF)
   { errmsg(169,rtnnme,mysection) ;
     return (mpsinINV) ; }
   tok = strtok(lex->string,sepchars) ;
@@ -312,10 +313,10 @@ static mpsinstate_enum mpsin_name (ioid mpschn, consys_struct **consys,
   It's an error to see anything else.
 */
   lex = getmpsline(mpschn) ;
-  if (lex->class == LCERR)
+  if (lex->class == DY_LCERR)
   { errmsg(168,rtnnme,mysection) ;
     return (mpsinINV) ; }
-  if (lex->class == LCEOF)
+  if (lex->class == DY_LCEOF)
   { errmsg(169,rtnnme,mysection) ;
     return (mpsinINV) ; }
   tok = strtok(lex->string,sepchars) ;
@@ -358,6 +359,11 @@ static mpsinstate_enum mpsin_rows (ioid mpschn, consys_struct *consys)
   bool seen_cols,keep_row ;
   pkvec_struct *pkrow ;
 
+  /*
+    Useful to suppress compiler warnings re. integer <-> pointer conversion
+  */
+  ptrdiff_t intermediary ;
+
   const char *rtnnme = "mpsin_rows",
 	     *mysection = "rows" ;
 
@@ -383,7 +389,7 @@ static mpsinstate_enum mpsin_rows (ioid mpschn, consys_struct *consys)
   seen_cols = FALSE ;
   consys->objndx = -1 ;
   for (lex = getmpsline(mpschn) ;
-       lex->class == LCQS ;
+       lex->class == DY_LCQS ;
        lex = getmpsline(mpschn))
   { tok = strtok(lex->string,sepchars) ;
     if (cistrcmp(tok,"columns") == 0)
@@ -446,8 +452,10 @@ static mpsinstate_enum mpsin_rows (ioid mpschn, consys_struct *consys)
 	if (consys->objnme == NULL) consys->objnme = STRALLOC(pkrow->nme) ; } }
     else
     { pkrow->ndx = -1 ; }
-    if ((void *) pkrow->ndx !=
-	enter(STRALLOC(pkrow->nme),conhash,conhashsze,(void *) pkrow->ndx))
+    intermediary = pkrow->ndx ;
+    if ((void *) intermediary !=
+	dyhash_enter(STRALLOC(pkrow->nme),conhash,conhashsze,
+		     (void *) intermediary))
     { errmsg(155,rtnnme,"constraint",
 	     consys_nme(consys,'c',pkrow->ndx,TRUE,NULL)) ;
       return (mpsinINV) ; }
@@ -469,10 +477,10 @@ static mpsinstate_enum mpsin_rows (ioid mpschn, consys_struct *consys)
 	      rtnnme,consys->archccnt) ;
 # endif
   if (seen_cols == FALSE)
-  { if (lex->class == LCERR)
+  { if (lex->class == DY_LCERR)
       errmsg(168,rtnnme,mysection) ;
     else
-    if (lex->class == LCEOF)
+    if (lex->class == DY_LCEOF)
       errmsg(169,rtnnme,mysection) ;
     else
       errmsg(1,rtnnme,__LINE__) ;
@@ -518,6 +526,11 @@ static mpsinstate_enum mpsin_columns (ioid mpschn, consys_struct *consys)
   char *rownam,*tok,*chkptr ;
   char colnam[50] ;		/* grossly oversized, but safe */
 
+  /*
+    Useful to suppress compiler warnings re. integer <-> pointer conversion
+  */
+  ptrdiff_t intermediary ;
+
   const char *rtnnme = "mpsin_columns",
 	     *mysection = "columns" ;
 
@@ -560,7 +573,7 @@ static mpsinstate_enum mpsin_columns (ioid mpschn, consys_struct *consys)
   vartype = vartypCON ;
 
   for (lex = getmpsline(mpschn) ;
-       lex->class == LCQS ;
+       lex->class == DY_LCQS ;
        lex = getmpsline(mpschn))
   { 
 /*
@@ -579,8 +592,10 @@ static mpsinstate_enum mpsin_columns (ioid mpschn, consys_struct *consys)
       { if (consys_addcol_pk(consys,vartype,pkcol,0.0,0.0,0.0) == FALSE)
 	{ errmsg(156,rtnnme,"column",consys->nme,pkcol->nme) ;
 	  return (mpsinINV) ; }
-	if ((void *) pkcol->ndx !=
-	    enter(STRALLOC(pkcol->nme),varhash,varhashsze,(void *) pkcol->ndx))
+	intermediary = pkcol->ndx ;
+	if ((void *) intermediary !=
+	    dyhash_enter(STRALLOC(pkcol->nme),varhash,varhashsze,
+	    	  (void *) intermediary))
 	{ errmsg(155,rtnnme,"variable",
 		 consys_nme(consys,'v',pkcol->ndx,TRUE,NULL)) ;
 	  return (mpsinINV) ; }
@@ -631,7 +646,9 @@ static mpsinstate_enum mpsin_columns (ioid mpschn, consys_struct *consys)
 	{ if (sosset == TRUE)
 	  { errmsg(161,rtnnme,"sosorg",pkcol->nme,"sosend") ;
 	    return (mpsinINV) ; }
-	  sosndx = (int) lookup(pkcol->nme,conhash,conhashsze) ;
+	  intermediary =
+		(ptrdiff_t) dyhash_lookup(pkcol->nme,conhash,conhashsze) ;
+	  sosndx = (int) intermediary ;
 	  if (sosndx == 0)
 	  { errmsg(162,rtnnme,"SOS constraint",pkcol->nme,"marker","sosorg") ;
 	    return (mpsinINV) ; }
@@ -643,7 +660,8 @@ static mpsinstate_enum mpsin_columns (ioid mpschn, consys_struct *consys)
 	  sos3 = (lnk_struct *) MALLOC(sizeof(lnk_struct)) ;
 	  sos3->llnxt = sos3lst ;
 	  sos3lst = sos3 ;
-	  lnk_in(sos3,sosndx) ; }
+	  intermediary = sosndx ;
+	  lnk_in(sos3,intermediary) ; }
 	else
 	if (cistrcmp(tok,"'sosend'") == 0)
 	{ if (sosset != TRUE)
@@ -704,7 +722,8 @@ static mpsinstate_enum mpsin_columns (ioid mpschn, consys_struct *consys)
       if (chkptr == tok || errno == ERANGE)
       { errmsg(165,rtnnme,tok,consys->nme,pkcol->nme,rownam) ;
 	return (mpsinINV) ; }
-      rowndx = (int) lookup(rownam,conhash,conhashsze) ;
+      intermediary = (ptrdiff_t) dyhash_lookup(rownam,conhash,conhashsze) ;
+      rowndx = (int) intermediary ;
       if (rowndx == 0)
       { errmsg(166,rtnnme,"constraint",rownam,"column",consys->nme,pkcol->nme) ;
 	return (mpsinINV) ; }
@@ -722,10 +741,10 @@ static mpsinstate_enum mpsin_columns (ioid mpschn, consys_struct *consys)
   the next state code.
 */
   if (nxtstate == mpsinINV)
-  { if (lex->class == LCERR)
+  { if (lex->class == DY_LCERR)
       errmsg(168,rtnnme,mysection) ;
     else
-    if (lex->class == LCEOF)
+    if (lex->class == DY_LCEOF)
       errmsg(169,rtnnme,mysection) ;
     else
       errmsg(1,rtnnme,__LINE__) ;
@@ -770,6 +789,10 @@ static mpsinstate_enum mpsin_rhs (ioid mpschn, consys_struct *consys)
   lex_struct *lex ;
   char *tok,*rownme,*chkptr ;
   const char *rhsnme ;
+  /*
+    Useful to suppress compiler warnings re. integer <-> pointer conversion
+  */
+  ptrdiff_t intermediary ;
 
   const char *rtnnme = "mpsin_rhs",
 	     *mysection = "rhs" ;
@@ -790,7 +813,7 @@ static mpsinstate_enum mpsin_rhs (ioid mpschn, consys_struct *consys)
   vector. An even number indicates the name is null.
 */
   lex = getmpsline(mpschn) ;
-  if (lex->class == LCQS)
+  if (lex->class == DY_LCQS)
   { rowndx = 1 ;
     for (tok = strpbrk(lex->string,sepchars) ;
 	 tok != NULL ;
@@ -813,7 +836,7 @@ static mpsinstate_enum mpsin_rhs (ioid mpschn, consys_struct *consys)
   three.
 */
   nxtstate = mpsinINV ;
-  for ( ; lex->class == LCQS ; lex = getmpsline(mpschn))
+  for ( ; lex->class == DY_LCQS ; lex = getmpsline(mpschn))
   { tok = strtok(lex->string,sepchars) ;
     if (cistrcmp(tok,"bounds") == 0) nxtstate = mpsinBOUNDS ;
     else
@@ -833,7 +856,8 @@ static mpsinstate_enum mpsin_rhs (ioid mpschn, consys_struct *consys)
   install the coefficient in the rhs vector.
 */
     for ( ; tok != NULL ; tok = strtok(NULL,sepchars))
-    { rowndx = (int) lookup(tok,conhash,conhashsze) ;
+    { intermediary = (ptrdiff_t) dyhash_lookup(tok,conhash,conhashsze) ;
+      rowndx = (int) intermediary ;
       if (rowndx == 0)
       { errmsg(166,rtnnme,"constraint",tok,consys_assocnme(NULL,CONSYS_RHS),
 	       consys->nme,rhsnme) ;
@@ -853,10 +877,10 @@ static mpsinstate_enum mpsin_rhs (ioid mpschn, consys_struct *consys)
   See if we're out of the loop due to an i/o error.
 */
   if (nxtstate == mpsinINV)
-  { if (lex->class == LCERR)
+  { if (lex->class == DY_LCERR)
       errmsg(168,rtnnme,mysection) ;
     else
-    if (lex->class == LCEOF)
+    if (lex->class == DY_LCEOF)
       errmsg(169,rtnnme,mysection) ;
     else
       errmsg(1,rtnnme,__LINE__) ;
@@ -906,6 +930,10 @@ static mpsinstate_enum mpsin_ranges (ioid mpschn, consys_struct *consys)
   lex_struct *lex ;
   const char *rngnme ;
   char *rownme,*tok,*chkptr ;
+  /*
+    Useful to suppress compiler warnings re. integer <-> pointer conversion
+  */
+  ptrdiff_t intermediary ;
 
   const char *rtnnme = "mpsin_ranges",
 	     *mysection = "ranges" ;
@@ -937,7 +965,7 @@ static mpsinstate_enum mpsin_ranges (ioid mpschn, consys_struct *consys)
   vector. An even number indicates the name is null.
 */
   lex = getmpsline(mpschn) ;
-  if (lex->class == LCQS)
+  if (lex->class == DY_LCQS)
   { rowndx = 1 ;
     for (tok = strpbrk(lex->string,sepchars) ;
 	 tok != NULL ;
@@ -959,7 +987,7 @@ static mpsinstate_enum mpsin_ranges (ioid mpschn, consys_struct *consys)
   optional section, so we look for both.
 */
   nxtstate = mpsinINV ;
-  for ( ; lex->class == LCQS ; lex = getmpsline(mpschn))
+  for ( ; lex->class == DY_LCQS ; lex = getmpsline(mpschn))
   { tok = strtok(lex->string,sepchars) ;
     if (cistrcmp(tok,"bounds") == 0) nxtstate = mpsinBOUNDS ;
     else
@@ -976,7 +1004,8 @@ static mpsinstate_enum mpsin_ranges (ioid mpschn, consys_struct *consys)
   to get the row index, then try for the coefficient.
 */
     for ( ; tok != NULL ; tok = strtok(NULL,sepchars))
-    { rowndx = (int) lookup(tok,conhash,conhashsze) ;
+    { intermediary = (ptrdiff_t) dyhash_lookup(tok,conhash,conhashsze) ;
+      rowndx = (int) intermediary ;
       if (rowndx == 0)
       { errmsg(166,rtnnme,"constraint",tok,"range vector",consys->nme,rngnme) ;
 	return (mpsinINV) ; }
@@ -1020,10 +1049,10 @@ static mpsinstate_enum mpsin_ranges (ioid mpschn, consys_struct *consys)
   See if we're out of the loop due to an i/o error.
 */
   if (nxtstate == mpsinINV)
-  { if (lex->class == LCERR)
+  { if (lex->class == DY_LCERR)
       errmsg(168,rtnnme,mysection) ;
     else
-    if (lex->class == LCEOF)
+    if (lex->class == DY_LCEOF)
       errmsg(169,rtnnme,mysection) ;
     else
       errmsg(1,rtnnme,__LINE__) ;
@@ -1084,6 +1113,10 @@ static mpsinstate_enum mpsin_bounds (ioid mpschn, consys_struct *consys)
   lex_struct *lex ;
   const char *bndnme ;
   char *varnme,*bndcode,*tok,*chkptr ;
+  /*
+    Useful to suppress compiler warnings re. integer <-> pointer conversion
+  */
+  ptrdiff_t intermediary ;
 
   const char *rtnnme = "mpsin_bounds",
 	     *mysection = "bounds" ;
@@ -1127,7 +1160,7 @@ static mpsinstate_enum mpsin_bounds (ioid mpschn, consys_struct *consys)
   we'll see 10 characters worth of space.
 */
   lex = getmpsline(mpschn) ;
-  if (lex->class == LCQS)
+  if (lex->class == DY_LCQS)
   { tok = (lex->string)+3 ;
     if (strspn(tok,sepchars) == 10)
     { named_vec = FALSE ;
@@ -1143,7 +1176,7 @@ static mpsinstate_enum mpsin_bounds (ioid mpschn, consys_struct *consys)
   and correct it below if we're wrong.
 */
   seen_end = FALSE ;
-  for ( ; lex->class == LCQS ; lex = getmpsline(mpschn))
+  for ( ; lex->class == DY_LCQS ; lex = getmpsline(mpschn))
   { tok = strtok(lex->string,sepchars) ;
     if (cistrcmp(tok,"endata") == 0)
     { seen_end = TRUE ;
@@ -1162,7 +1195,8 @@ static mpsinstate_enum mpsin_bounds (ioid mpschn, consys_struct *consys)
     if (varnme == NULL)
     { errmsg(153,rtnnme,"variable",mysection) ;
       return (mpsinINV) ; }
-    colndx = (int) lookup(varnme,varhash,varhashsze) ;
+    intermediary = (ptrdiff_t) dyhash_lookup(varnme,varhash,varhashsze) ;
+    colndx = (int) intermediary ;
     if (colndx == 0)
     { errmsg(162,rtnnme,"variable",varnme,consys->nme,mysection) ;
       return (mpsinINV) ; }
@@ -1238,10 +1272,10 @@ static mpsinstate_enum mpsin_bounds (ioid mpschn, consys_struct *consys)
   Check to see if we escaped the loop due to i/o error.
 */
   if (seen_end == FALSE)
-  { if (lex->class == LCERR)
+  { if (lex->class == DY_LCERR)
       errmsg(168,rtnnme,mysection) ;
     else
-    if (lex->class == LCEOF)
+    if (lex->class == DY_LCEOF)
       errmsg(169,rtnnme,mysection) ;
     else
       errmsg(1,rtnnme,__LINE__) ;
@@ -1287,6 +1321,10 @@ static mpsinstate_enum mpsin_enddata (ioid mpschn, consys_struct *consys,
   pkvec_struct *pkvec ;
   hel *entry ;
   lex_struct *lex ;
+  /*
+    Useful to suppress compiler warnings re. integer <-> pointer conversion
+  */
+  ptrdiff_t intermediary ;
 
   const char *rtnnme = "mpsin_enddata" ;
 
@@ -1378,7 +1416,8 @@ static mpsinstate_enum mpsin_enddata (ioid mpschn, consys_struct *consys,
   Do some integrity checks while we're at it, provided debugging is enabled.
 */
   for (sos3 = sos3lst ; sos3 != NULL ; sos3 = sos3->llnxt)
-  { ndx = lnk_out(sos3,int) ;
+  { intermediary = lnk_out(sos3,ptrdiff_t) ;
+    ndx = (int) intermediary ;
 #   ifndef NDEBUG
     if (ndx < 1 || ndx > consys->archccnt)
     { errmsg(102,rtnnme,consys->nme,"constraint",ndx,1,consys->archccnt) ;
@@ -1446,8 +1485,8 @@ static mpsinstate_enum mpsin_enddata (ioid mpschn, consys_struct *consys,
 /*
   Clean up after dyio_scanstr.
 */
-  lex = dyio_scanstr(mpschn,LCQS,0,'\0','\0') ;
-  if (lex->class == LCQS && lex->string != NULL)
+  lex = dyio_scanstr(mpschn,DY_LCQS,0,'\0','\0') ;
+  if (lex->class == DY_LCQS && lex->string != NULL)
   { FREE(lex->string) ;
     lex->string = NULL ; }
   
