@@ -38,7 +38,7 @@
 #include "dylp.h"
 
 static char sccsid[] UNUSED = "@(#)dy_primalmultipivot.c	1.4	11/06/04" ;
-static char svnid[] UNUSED = "$Id: dy_primalmultipivot.c 183 2007-10-05 00:54:29Z lou $" ;
+static char svnid[] UNUSED = "$Id: dy_primalmultipivot.c 340 2010-07-24 03:34:28Z lou $" ;
 
 /*
   The structure to hold pivot candidates
@@ -291,18 +291,18 @@ static dyret_enum scanForPrimOutCands (primcand_struct *outcands,
 */
   const bool allowsoftdegen = FALSE ;
 
-# ifdef PARANOIA
+# ifdef DYLP_PARANOIA
   const char *rtnnme = "scanForPrimOutCands" ;
 # endif
 
-# if !defined(DYLP_NDEBUG) || defined(PARANOIA)
+# if !defined(DYLP_NDEBUG) || defined(DYLP_PARANOIA)
   int print ;
 
   print = dy_opts->print.pivoting ;	/* suppress print in dy_chkpiv */
   dy_opts->print.pivoting = 0 ;
 # endif
 
-# ifdef PARANOIA
+# ifdef DYLP_PARANOIA
   if (outcands == NULL)
   { errmsg(2,rtnnme,"outcands array") ;
     return (dyrFATAL) ; }
@@ -369,7 +369,7 @@ static dyret_enum scanForPrimOutCands (primcand_struct *outcands,
   lastcandcnt = candcnt ;
   for (kpos = 1 ; kpos <= m ; kpos++)
   { k = dy_basis[kpos] ;
-#   ifdef PARANOIA
+#   ifdef DYLP_PARANOIA
     if (dy_chkstatus(k) == FALSE)
     { outcands[0].ndx = -1 ;
       dy_opts->print.pivoting = print ;
@@ -779,6 +779,10 @@ dyret_enum primmultiout (int j, int indir,
 
   primcand_struct *outcands,*candk ;
 
+# if MALLOC_DEBUG == 2
+  char *rtnnme = "dy_primmultiout" ;
+# endif
+
 /*
   Setup. Potentially, each basic variable can produce a soft and a hard pivot
   delta, and we need to allow for a bound-to-bound pivot. Hence we need a
