@@ -16,7 +16,7 @@
 
 /*
   @(#)dylp.h	4.6	10/15/05
-  svn/cvs: $Id: dylp.h 218 2008-03-28 15:57:22Z lou $
+  svn/cvs: $Id: dylp.h 365 2010-08-29 01:22:48Z lou $
 
   This file contains definitions related to dylp, a subroutine library which
   implements a dynamic (primal-dual) linear programming algorithm based on
@@ -534,6 +534,7 @@ typedef struct
 
   Field		Definition
   -----		----------
+  owner		ID of the owner of this problem.
   ctlopts	Control and status flags.		
   phase		(i) If set to dyDONE, dylp will free any retained data
 		    structures and return. Any other value is ignored.
@@ -545,10 +546,13 @@ typedef struct
 		For lpINFEAS, the total infeasibility.
 		For lpUNBOUNDED, the index of the unbounded variable, negated
 		  if the variable can decrease without bound, positive if it
-		  can increase without bound.
+		  can increase without bound. The logical for constraint i
+		  is represented as n+i.
 		Otherwise, undefined.
   iters		The number of simplex iterations.
   consys	The constraint system.
+  fullsys	True if the active constraint system is the full system, false
+  		otherwise. 
   basis		(i) Initial basis.
   		(o) Final basis.
   status	(i) Initial status vector.
@@ -580,12 +584,14 @@ typedef struct
 */
 
 typedef struct
-{ flags ctlopts ;
+{ void *owner ;
+  flags ctlopts ;
   dyphase_enum phase ;
   lpret_enum lpret ;
   double obj ;
   int iters ;
   consys_struct *consys ;
+  bool fullsys ;
   basis_struct *basis ;
   flags *status ;
   double *x ;
@@ -762,18 +768,26 @@ typedef enum { ibINV = 0, ibLOGICAL, ibSLACK, ibARCH } ibtype_enum ;
 /*
   Enum for calling context.
 
-  As dylp evolves, it may well prove useful to know the context of the
-  call. Consider this an experiment. The default context is INITIALLP.
+  As dylp evolves, it has proven useful to know the context of the
+  call. Consider this a work in progress. The default context is INITIALLP.
 
   cxINV		invalid (context is unknown)
+  cxLOAD	This call is only to (re)load data structures. Returns after
+		one iteration of dual or primal simplex, but shows problem
+		status rather than lpITERLIM.
+  cxUNLOAD	This call is solely for the purpose of freeing the problem
+		data structures. (Replaces dyDONE/lpctlONLYFREE hack.)
   cxSINGLELP	This is a one-off call to solve a single LP from scratch.
   cxINITIALLP	This is a call to solve a single LP from scratch, but will
 		likely be followed by calls to reoptimise.
   cxBANDC	This call is made in the context of a branch-and-cut
 		algorithm.
+  cxUSERPIV	This call is in the context of user control of pivoting.
 */
 
-typedef enum { cxINV = 0, cxSINGLELP, cxINITIALLP, cxBANDC } cxtype_enum ;
+typedef enum { cxINV = 0, cxLOAD, cxUNLOAD,
+	       cxSINGLELP, cxINITIALLP, cxBANDC, cxUSERPIV } cxtype_enum ;
+
 
 /*
   lpopts_struct
@@ -896,7 +910,7 @@ typedef enum { cxINV = 0, cxSINGLELP, cxINITIALLP, cxBANDC } cxtype_enum ;
 		  0:	catatonic
 		  1:	issue a warning
 		  2:	issue an error message and force an abort
-		Numeric codes are related to keywords in dy_setup.c:dy_ctlopt.
+		Numeric codes should match keywords in dy_options.c:dy_ctlopt.
   degen		TRUE to allow creation of restricted subproblems to deal with
 		degeneracy, FALSE to disallow it.
   degenpivlim	The number of successive degenerate pivots required before
@@ -922,7 +936,7 @@ typedef enum { cxINV = 0, cxSINGLELP, cxINITIALLP, cxBANDC } cxtype_enum ;
 		     hyperplane which will become tight on the pivot; choose
 		     so that the normal of the hyperplane is most nearly
 		     aligned with the direction of motion
-		Numeric codes are related to keywords in dy_setup.c:dy_ctlopt.
+		Numeric codes should match keywords in dy_options.c:dy_ctlopt.
   patch		TRUE to allow the code to patch a singular basis, FALSE to
 		prevent patching.
   copyorigsys	Controls whether dylp makes a local copy of the original
@@ -930,12 +944,15 @@ typedef enum { cxINV = 0, cxSINGLELP, cxINITIALLP, cxBANDC } cxtype_enum ;
 		If set to FALSE, a copy will be made only if necessary.
 		Scaling will trigger a local copy.
   scaling	Controls whether dylp attempts to scale the original constraint
-		system.
+		system for numeric stability.
 		  0: scaling is forbidden
-		  1: scale the original constraint system using scaling vectors
-		     attached to the system
+		  1: scale the original constraint system using numeric
+		     scaling vectors attached to the system
 		  2: evaluate the original constraint system and scale it if
 		     necessary
+		Note that even if scaling = 0, dylp may install +/-1.0 scaling
+		vectors in order to flip >= constraints to <= constraints. See
+		comments in dy_scaling.c
   print		Substructure for picky printing control. For all print options,
 		a value of 0 suppresses all information messages.
     major	  Controls printing of major phase information.
@@ -1072,6 +1089,26 @@ typedef enum { cxINV = 0, cxSINGLELP, cxINITIALLP, cxBANDC } cxtype_enum ;
 		     constraints and variables.
 		  3: additional information about variables and constraints
 		     examined.
+    tableau	  Controls print level for routines that generate tableau
+		  vectors (beta<i>, beta<j>, abar<i>, abar<j>) for use by
+		  external clients.
+		  1: prints summary messages about the circumstances
+		  4: prints nonzeros in the final vector.
+		  5: prints nonzeros in intermediate vectors and (dy_betaj,
+		     dy_abarj only) inactive rows
+		  6: prints nonzeros of active portion in internal reference
+		     frame (dy_betaj only)
+    rays	  Controls print level for routines that generate primal
+		  and dual rays for use by external clients.
+		  1: prints summary messages about vectors found.
+		  3: print information about columns / rows examined.
+		  4: print information about why a column or row was rejected.
+		  5: print nonzeros for each ray
+    soln	  Controls print level for routines that generate primal and
+		  dual solutions for use by external clients.
+		  1: prints summary messages about the circumstances
+		  3: prints nonzeros in the final vector
+		  4: prints nonzeros in intermediate vectors
 */
 
 typedef struct
@@ -1134,197 +1171,200 @@ typedef struct
 	   int basis ;
 	   int conmgmt ;
 	   int varmgmt ;
-	   int force ; } print ; } lpopts_struct ;
+	   int force ;
+	   int tableau ;
+	   int rays ;
+	   int soln ; } print ; } lpopts_struct ;
 
 
-
+  
 
-/*
-  Statistics structure, used to collect information about aspects of dylp
-  operation beyond simple pivot counts. The data structure definition is
-  always present, but to fill it you have to define DYLP_STATISTICS.
+  /*
+    Statistics structure, used to collect information about aspects of dylp
+    operation beyond simple pivot counts. The data structure definition is
+    always present, but to fill it you have to define DYLP_STATISTICS.
 
-  Field		Definition
-  -----		----------
-  phasecnts[dyDONE] Array with counts for number of times each phase is
-		executed.
-  ini_simplex	The initial simplex phase
-  cons		A set of arrays with data about individual constraints.
-    sze		  Allocated capacity of the arrays.
-    angle	  Angle to the objective function.
-    actcnt	  Number of times constraint is activated.
-    deactcnt	  Number of times constraint is deactivated.
-    init	  True if constraint is active in initial system.
-    fin		  True if constraint is active in final system.
-  vars		A set of arrays with data about individual variables.
-    sze		  Allocated capacity of the arrays.
-    actcnt	  Number of times variable is activated.
-    deactcnt	  Number of times variable is deactivated.
-  angle
-    max		Maximum angle to the objective function over all constraints.
-    min		Minimum angle to the objective function over all constraints.
-    hist[*]	Histogram of angles of constraints to the objective function.
-		There are DYSTATS_HISTBINS bins. Currently, 37 bins: 36 bins
-		spanning 5 degrees of angle, and a dedicated 90 degree bin.
-  
-  factor	Tracks how well we're doing with respect to refactoring the
-		basis.
-    cnt		Number of time the basis has been refactored.
-    prevpiv	Pivot count at last refactorisation.
-    avgpivs	Average number of pivots between basis refactorisations.
-    maxpivs	Maximum number of pivots between basis refactorisations.
+    Field		Definition
+    -----		----------
+    phasecnts[dyDONE] Array with counts for number of times each phase is
+		  executed.
+    ini_simplex	The initial simplex phase
+    cons		A set of arrays with data about individual constraints.
+      sze		  Allocated capacity of the arrays.
+      angle	  Angle to the objective function.
+      actcnt	  Number of times constraint is activated.
+      deactcnt	  Number of times constraint is deactivated.
+      init	  True if constraint is active in initial system.
+      fin		  True if constraint is active in final system.
+    vars		A set of arrays with data about individual variables.
+      sze		  Allocated capacity of the arrays.
+      actcnt	  Number of times variable is activated.
+      deactcnt	  Number of times variable is deactivated.
+    angle
+      max		Maximum angle to the objective function over all constraints.
+      min		Minimum angle to the objective function over all constraints.
+      hist[*]	Histogram of angles of constraints to the objective function.
+		  There are DYSTATS_HISTBINS bins. Currently, 37 bins: 36 bins
+		  spanning 5 degrees of angle, and a dedicated 90 degree bin.
+    
+    factor	Tracks how well we're doing with respect to refactoring the
+		  basis.
+      cnt		Number of time the basis has been refactored.
+      prevpiv	Pivot count at last refactorisation.
+      avgpivs	Average number of pivots between basis refactorisations.
+      maxpivs	Maximum number of pivots between basis refactorisations.
 
-  pivrej	Statistics about the pivot rejection list and punts.
-    max		  maximum number of entries on the pivot rejection list
-    mad		  total number of entries attributed to mad pivots
-    sing	  total number of entries attributed to singular pivots
-    pivtol_red	  total number of times the pivot tolerance was reduced
-    min_pivtol	  the minimum pivot tolerance used
-    puntcall	  total number of calls to dealWithPunt
-    puntret	  total number of dyrPUNT returns recommended
+    pivrej	Statistics about the pivot rejection list and punts.
+      max		  maximum number of entries on the pivot rejection list
+      mad		  total number of entries attributed to mad pivots
+      sing	  total number of entries attributed to singular pivots
+      pivtol_red	  total number of times the pivot tolerance was reduced
+      min_pivtol	  the minimum pivot tolerance used
+      puntcall	  total number of calls to dealWithPunt
+      puntret	  total number of dyrPUNT returns recommended
 
-  dmulti	Tracks the dual multipivot algorithm. All fields except cnt are
-		totals; divide by cnt to get averages.
-    flippable	  Number of flippable variables in the constraint system.
-    cnt		  Total calls to dualmultiin
-    cands	  Number of candidates queued for evaluation for entry
-    promote	  Number of calls that resulted in promoting a sane pivot
-		  over an unstable pivot.
-    nontrivial	  Number of times that the initial scan and sort left
-		  multiple candidates for further evaluation.
-    evals	  Actual number of candidates evaluated (ftran'd column)
-    flips	  Number of bound-to-bound flips performed
-    pivrnk	  Index in the list of candidates of the candidate finally
-		  selected for pivoting.
-    maxrnk	  Maximum index selected for pivoting.
+    dmulti	Tracks the dual multipivot algorithm. All fields except cnt are
+		  totals; divide by cnt to get averages.
+      flippable	  Number of flippable variables in the constraint system.
+      cnt		  Total calls to dualmultiin
+      cands	  Number of candidates queued for evaluation for entry
+      promote	  Number of calls that resulted in promoting a sane pivot
+		    over an unstable pivot.
+      nontrivial	  Number of times that the initial scan and sort left
+		    multiple candidates for further evaluation.
+      evals	  Actual number of candidates evaluated (ftran'd column)
+      flips	  Number of bound-to-bound flips performed
+      pivrnk	  Index in the list of candidates of the candidate finally
+		    selected for pivoting.
+      maxrnk	  Maximum index selected for pivoting.
 
-  pmulti	Tracks the primal multipivot algorithm.
-    cnt		  Total calls to primalmultiin
-    cands	  Number of candidates queued for evaluation to leave
-    nontrivial	  Number of times that the candidate list was sorted
-    promote	  Number of calls that resulted in promoting a sane pivot
-		  over an unstable pivot.
-  
-  infeas	Statistics on resolution of infeasibility in primal phase I.
-		Basically, what we're interested in tracking is the number
-		of infeasible variables and the number of pivots between a
-		change in the number of infeasible variables. We're interested
-		in separating the case of 1 variable from 2 or more, because
-		the latter requires vastly more calculation. A little care
-		is required because phase I can run many times.
+    pmulti	Tracks the primal multipivot algorithm.
+      cnt		  Total calls to primalmultiin
+      cands	  Number of candidates queued for evaluation to leave
+      nontrivial	  Number of times that the candidate list was sorted
+      promote	  Number of calls that resulted in promoting a sane pivot
+		    over an unstable pivot.
+    
+    infeas	Statistics on resolution of infeasibility in primal phase I.
+		  Basically, what we're interested in tracking is the number
+		  of infeasible variables and the number of pivots between a
+		  change in the number of infeasible variables. We're interested
+		  in separating the case of 1 variable from 2 or more, because
+		  the latter requires vastly more calculation. A little care
+		  is required because phase I can run many times.
 
-    prevpiv	The pivot count (tot.iters) at the previous change.
-    maxcnt	The maximum number of infeasible variables encountered (this
-		is not strictly monotonic, as dylp may enter phase I many
-		times due to activating violated constraints).
-    totpivs	The total number of pivots expended in phase I.
-    maxpivs	The maximum number of pivots with no change in the number of
-		feasible variables.
-    chgcnt1	The number of times that the number of infeasible variables
-		changed and reduced costs did not have to be recalculated
-		(specifically, exactly one variable became feasible, and it
-		left the basis as it did so).
-    chgcnt2	The number of times that the number of infeasible variables
-		changed in such a way as to require recalculation of the
-		reduced costs.
+      prevpiv	The pivot count (tot.iters) at the previous change.
+      maxcnt	The maximum number of infeasible variables encountered (this
+		  is not strictly monotonic, as dylp may enter phase I many
+		  times due to activating violated constraints).
+      totpivs	The total number of pivots expended in phase I.
+      maxpivs	The maximum number of pivots with no change in the number of
+		  feasible variables.
+      chgcnt1	The number of times that the number of infeasible variables
+		  changed and reduced costs did not have to be recalculated
+		  (specifically, exactly one variable became feasible, and it
+		  left the basis as it did so).
+      chgcnt2	The number of times that the number of infeasible variables
+		  changed in such a way as to require recalculation of the
+		  reduced costs.
 
-  [dp]degen[*]	Array of stats for each restricted subproblem nesting level,
-		with separate arrays for dual (ddegen) and primal (pdegen).
-		degen[0].cnt is used to hold the maximum nesting level.
-    cnt		Number of times this nesting level was entered.
-    avgsiz	The average number of variables in a restricted subproblem.
-		Kept by iterative update, as avg<k+1> = (avg<k>*k+size)/(k+1).
-		Suffers from cumulative loss of accuracy, but it'll do for
-		our purposes.
-    maxsiz	The maximum number of variables in a restricted subproblem.
-    totpivs	Total number of pivots at or above this nesting level.
-    avgpivs	Average number of pivots at or above this nesting level.
-    maxpivs	Maximum number of pivots for any one instance at or above
-		this nesting level.
+    [dp]degen[*]	Array of stats for each restricted subproblem nesting level,
+		  with separate arrays for dual (ddegen) and primal (pdegen).
+		  degen[0].cnt is used to hold the maximum nesting level.
+      cnt		Number of times this nesting level was entered.
+      avgsiz	The average number of variables in a restricted subproblem.
+		  Kept by iterative update, as avg<k+1> = (avg<k>*k+size)/(k+1).
+		  Suffers from cumulative loss of accuracy, but it'll do for
+		  our purposes.
+      maxsiz	The maximum number of variables in a restricted subproblem.
+      totpivs	Total number of pivots at or above this nesting level.
+      avgpivs	Average number of pivots at or above this nesting level.
+      maxpivs	Maximum number of pivots for any one instance at or above
+		  this nesting level.
 
-  tot, p1, p2, d2	Iteration and pivot counts, total and for each
-			individual phase. These are copied over from
-			dy_lp (lp_struct) at the end of the run, so that
-			they can be printed by dumpstats.
-  
-  DYSTATS_MAXDEGEN is the maximum number of levels of nesting accommodated by
-  antidegeneracy statistics and debugging structures. The actual algorithm
-  has no inherent limitation.
+    tot, p1, p2, d2	Iteration and pivot counts, total and for each
+			  individual phase. These are copied over from
+			  dy_lp (lp_struct) at the end of the run, so that
+			  they can be printed by dumpstats.
+    
+    DYSTATS_MAXDEGEN is the maximum number of levels of nesting accommodated by
+    antidegeneracy statistics and debugging structures. The actual algorithm
+    has no inherent limitation.
 
-  DYSTATS_HISTBINS is the number of bins for constraint angles. It should be an
-  odd number. Each bin will span 180/(DYSTATS_HISTBINS-1) degrees, with the
-  final bin reserved for constraints at 90 degrees. For example, a value of 37
-  gives 180/(37-1) = 5 degrees per bin.
-*/
+    DYSTATS_HISTBINS is the number of bins for constraint angles. It should be an
+    odd number. Each bin will span 180/(DYSTATS_HISTBINS-1) degrees, with the
+    final bin reserved for constraints at 90 degrees. For example, a value of 37
+    gives 180/(37-1) = 5 degrees per bin.
+  */
 
-#define DYSTATS_MAXDEGEN 25
-#define DYSTATS_HISTBINS 37
+  #define DYSTATS_MAXDEGEN 25
+  #define DYSTATS_HISTBINS 37
 
-typedef struct {
-  int phasecnts[dyDONE+1] ;
-  dyphase_enum ini_simplex ;
-  struct { int sze ;
-	   double *angle ;
-	   int *actcnt ;
-	   int *deactcnt ;
-	   bool *init ;
-	   bool *fin ; } cons ;
-  struct { int sze ;
-	   int *actcnt ;
-	   int *deactcnt ; } vars ;
-  struct { float max ;
-	   float min ;
-	   int hist[DYSTATS_HISTBINS] ; } angle ;
-  struct { int cnt ;
-	   int prevpiv ;
-	   float avgpivs ;
-	   int maxpivs ; } factor ;
-  struct { int max ;
-	   int mad ;
-	   int sing ;
-	   int pivtol_red ;
-	   double min_pivtol ;
-	   int puntcall ;
-	   int puntret ; } pivrej ;
-  struct { int flippable ;
-	   int cnt ;
-	   int cands ;
-	   int promote ;
-	   int nontrivial ;
-	   int evals ;
-	   int flips ;
-	   int pivrnks ;
-	   int maxrnk ; } dmulti ;
-  struct { int cnt ;
-	   int cands ;
-	   int nontrivial ;
-	   int promote ; } pmulti ;
-  struct { int prevpiv ;
-	   int maxcnt ;
-	   int totpivs ;
-	   int maxpivs ;
-	   int chgcnt1 ;
-	   int chgcnt2 ; } infeas ;
-  struct { int cnt ;
-	   float avgsiz ;
-	   int maxsiz ;
-	   int totpivs ;
-	   float avgpivs ;
-	   int maxpivs ; } pdegen[DYSTATS_MAXDEGEN] ;
-  struct { int cnt ;
-	   float avgsiz ;
-	   int maxsiz ;
-	   int totpivs ;
-	   float avgpivs ;
-	   int maxpivs ; } ddegen[DYSTATS_MAXDEGEN] ;
-  struct { int iters ;
-	   int pivs ; } tot ;
-  struct { int iters ;
-	   int pivs ; } p1 ;
-  struct { int iters ;
-	   int pivs ; } p2 ;
-  struct { int iters ;
-	   int pivs ; } d2 ; } lpstats_struct ;
+  typedef struct {
+    int phasecnts[dyDONE+1] ;
+    dyphase_enum ini_simplex ;
+    struct { int sze ;
+	     double *angle ;
+	     int *actcnt ;
+	     int *deactcnt ;
+	     bool *init ;
+	     bool *fin ; } cons ;
+    struct { int sze ;
+	     int *actcnt ;
+	     int *deactcnt ; } vars ;
+    struct { float max ;
+	     float min ;
+	     int hist[DYSTATS_HISTBINS] ; } angle ;
+    struct { int cnt ;
+	     int prevpiv ;
+	     float avgpivs ;
+	     int maxpivs ; } factor ;
+    struct { int max ;
+	     int mad ;
+	     int sing ;
+	     int pivtol_red ;
+	     double min_pivtol ;
+	     int puntcall ;
+	     int puntret ; } pivrej ;
+    struct { int flippable ;
+	     int cnt ;
+	     int cands ;
+	     int promote ;
+	     int nontrivial ;
+	     int evals ;
+	     int flips ;
+	     int pivrnks ;
+	     int maxrnk ; } dmulti ;
+    struct { int cnt ;
+	     int cands ;
+	     int nontrivial ;
+	     int promote ; } pmulti ;
+    struct { int prevpiv ;
+	     int maxcnt ;
+	     int totpivs ;
+	     int maxpivs ;
+	     int chgcnt1 ;
+	     int chgcnt2 ; } infeas ;
+    struct { int cnt ;
+	     float avgsiz ;
+	     int maxsiz ;
+	     int totpivs ;
+	     float avgpivs ;
+	     int maxpivs ; } pdegen[DYSTATS_MAXDEGEN] ;
+    struct { int cnt ;
+	     float avgsiz ;
+	     int maxsiz ;
+	     int totpivs ;
+	     float avgpivs ;
+	     int maxpivs ; } ddegen[DYSTATS_MAXDEGEN] ;
+    struct { int iters ;
+	     int pivs ; } tot ;
+    struct { int iters ;
+	     int pivs ; } p1 ;
+    struct { int iters ;
+	     int pivs ; } p2 ;
+    struct { int iters ;
+	     int pivs ; } d2 ; } lpstats_struct ;
 
 
 
@@ -1569,6 +1609,10 @@ typedef struct
   unpacking a structure on a regular basis. Unless otherwise indicated, indices
   are in the dy_sys (active system) frame of reference.
 
+  dy_owner	Null if there's no active problem. Contains the ID of the
+  		current owner if there's an active problem. Passed in as
+		part of the lpprob_struct.
+
   Main structures
   ---------------
   dy_lp:	The lp control structure for dylp.
@@ -1669,6 +1713,8 @@ typedef struct
 		0, the constraint is not involved in degeneracy.
 */
 
+extern void *dy_owner ;
+
 extern lp_struct *dy_lp ;
 extern consys_struct *dy_sys ;
 extern lptols_struct *dy_tols ;
@@ -1689,6 +1735,8 @@ extern lpstats_struct *dy_stats ;
 extern bool dy_initlclsystem(lpprob_struct *orig_lp, bool hotstart) ;
 extern void dy_freelclsystem(lpprob_struct *orig_lp, bool freesys) ;
 extern bool dy_isscaled(void) ;
+extern void dy_scaling_vectors(const double **rscale, const double **cscale) ;
+extern consys_struct *dy_scaled_origsys() ;
 
 /*
   dy_coldstart.c
@@ -1825,6 +1873,7 @@ extern void dy_defaults(lpopts_struct **opts, lptols_struct **tols),
 
 extern lpret_enum dylp(lpprob_struct *orig_lp, lpopts_struct *orig_opts,
 		       lptols_struct *orig_tols, lpstats_struct *orig_stats) ;
+extern void *dy_getOwner() ;
 
 /*
   dylp_utils.c
@@ -1841,7 +1890,7 @@ extern void dy_calcduals(void),dy_setbasicstatus(void),
 extern double dy_calcobj(void),dy_calcdualobj(void),dy_calcpinfeas(void) ;
 extern void dy_finishup(lpprob_struct *orig_lp, dyphase_enum phase) ;
 
-#ifdef PARANOIA
+#ifdef DYLP_PARANOIA
 
 extern bool dy_chkstatus(int vndx),
             dy_chkdysys(consys_struct *orig_sys) ;
@@ -1854,8 +1903,7 @@ extern void dy_chkdual(int lvl) ;
 extern bool dy_dupbasis(int dst_basissze, basis_struct **p_dst_basis,
 			basis_struct *src_basis, int dst_statussze,
 			flags **p_dst_status,
-			int src_statuslen, flags *src_status),
-	    dy_expandxopt(lpprob_struct *lp, double **p_xopt) ;
+			int src_statuslen, flags *src_status) ;
 extern void dy_freesoln(lpprob_struct *lpprob) ;
 
 /*
@@ -1869,6 +1917,47 @@ extern bool dy_pricenbvars(lpprob_struct *orig_lp, flags priceme,
 			    int nbcnt, int *nbvars,
 			    double *cbar, double *p_upeni, double *p_dpeni) ;
 
+/*
+  dy_tableau.c
+*/
+
+extern bool dy_abarj(lpprob_struct *orig_lp, int tgt_j, double **p_abarj) ;
+extern bool dy_betaj(lpprob_struct *orig_lp, int tgt_j, double **p_betaj) ;
+extern bool dy_betak(lpprob_struct *orig_lp, int col_k, double **p_betaj) ;
+extern bool dy_betai(lpprob_struct *orig_lp, int tgt_i, double **p_betai) ;
+extern bool dy_abari(lpprob_struct *orig_lp, int tgt_i, double **p_abari,
+		     double **p_betai) ;
+
+/*
+  dy_rays.c
+*/
+
+extern bool dy_primalRays(lpprob_struct *orig_lp,
+			  int *p_numRays, double ***p_rays) ;
+extern bool dy_dualRays(lpprob_struct *orig_lp, bool fullRay,
+			int *p_numRays, double ***p_rays, bool trueDuals) ;
+
+/*
+  dy_solutions.c
+*/
+
+extern void dy_colDuals(lpprob_struct *orig_lp, double **p_cbar,
+			bool trueDuals) ;
+extern void dy_rowDuals(lpprob_struct *orig_lp, double **p_y,
+			bool trueDuals) ;
+extern void dy_rowDualsGivenC(lpprob_struct *orig_lp, double **p_y,
+			      const double *c, bool trueDuals) ;
+
+extern void dy_colPrimals(lpprob_struct *orig_lp, double **p_x) ;
+extern void dy_rowPrimals(lpprob_struct *orig_lp,
+			  double **p_xB, int **p_indB) ;
+extern void dy_logPrimals(lpprob_struct *orig_lp, double **p_logx) ;
+
+extern void dy_colStatus(lpprob_struct *orig_lp, flags **p_colstat) ;
+extern void dy_logStatus(lpprob_struct *orig_lp, flags **p_logstat) ;
+
+extern bool dy_expandxopt(lpprob_struct *lp, double **p_xopt) ;
+
 /*
   dylp_io.c
 */
@@ -1888,6 +1977,8 @@ extern const char *dy_prtlpret(lpret_enum lpret),
 extern char *dy_prtvstat(flags status) ;
 extern bool dy_dumpcompact(ioid chn, bool echo, lpprob_struct *soln,
 			   bool nbzeros) ;
+extern void dy_setlogchn (ioid chn) ;
+extern void dy_setgtxecho (bool echo) ;
 
 /*
   dy_statistics.c
