@@ -1,8 +1,8 @@
-// Copyright (C) 2004, 2008 International Business Machines and others.
+// Copyright (C) 2004, 2010 International Business Machines and others.
 // All Rights Reserved.
 // This code is published under the Common Public License.
 //
-// $Id: IpOrigIpoptNLP.cpp 1219 2008-04-25 18:14:35Z andreasw $
+// $Id: IpOrigIpoptNLP.cpp 1665 2010-04-28 16:28:20Z andreasw $
 //
 // Authors:  Carl Laird, Andreas Waechter     IBM    2004-08-13
 
@@ -11,6 +11,16 @@
 #include "IpIpoptData.hpp"
 #include "IpIpoptCalculatedQuantities.hpp"
 
+#ifdef HAVE_CSTDIO
+# include <cstdio>
+#else
+# ifdef HAVE_STDIO_H
+#  include <stdio.h>
+# else
+#  error "don't have header file for stdio"
+# endif
+#endif
+
 namespace Ipopt
 {
 #if COIN_IPOPT_VERBOSITY > 0
@@ -80,7 +90,10 @@ namespace Ipopt
       "Activating this option will cause an error if an invalid number is "
       "detected in the constraint Jacobians or the Lagrangian Hessian.  If "
       "this is not activated, the test is skipped, and the algorithm might "
-      "proceed with invalid numbers and fail.");
+      "proceed with invalid numbers and fail.  If test is activated and an "
+      "invalid number is detected, the matrix is written to output with "
+      "print_level corresponding to J_MORE_DETAILED; so beware of large "
+      "output!");
     roptions->AddStringOption2(
       "jac_c_constant",
       "Indicates whether all equality constraints are linear",
@@ -294,7 +307,7 @@ namespace Ipopt
 
       if (x_space_->Dim() < c_space_->Dim()) {
         char msg[128];
-        sprintf(msg,"Too few degrees of freedom: %d equality constriants but only %d variables", c_space_->Dim(), x_space_->Dim());
+        Snprintf(msg, 127, "Too few degrees of freedom: %d equality constriants but only %d variables", c_space_->Dim(), x_space_->Dim());
         THROW_EXCEPTION(TOO_FEW_DOF, msg);
       }
       ASSERT_EXCEPTION(x_space_->Dim() > 0, TOO_FEW_DOF,
@@ -373,14 +386,14 @@ namespace Ipopt
     d_L_ = NLP_scaling()->apply_vector_scaling_d_LU(*Pd_L_, d_L_, *d_space_);
     d_U_ = NLP_scaling()->apply_vector_scaling_d_LU(*Pd_U_, d_U_, *d_space_);
 
-    x_L->Print(*jnlst_, J_VECTOR, J_INITIALIZATION,
-               "modified x_L scaled");
-    x_U->Print(*jnlst_, J_VECTOR, J_INITIALIZATION,
-               "modified x_U scaled");
-    d_L->Print(*jnlst_, J_VECTOR, J_INITIALIZATION,
-               "modified d_L scaled");
-    d_U->Print(*jnlst_, J_VECTOR, J_INITIALIZATION,
-               "modified d_U scaled");
+    x_L_->Print(*jnlst_, J_VECTOR, J_INITIALIZATION,
+                "modified x_L scaled");
+    x_U_->Print(*jnlst_, J_VECTOR, J_INITIALIZATION,
+                "modified x_U scaled");
+    d_L_->Print(*jnlst_, J_VECTOR, J_INITIALIZATION,
+                "modified d_L scaled");
+    d_U_->Print(*jnlst_, J_VECTOR, J_INITIALIZATION,
+                "modified d_U scaled");
 
     // Create the iterates structures
     x = x_space_->MakeNew();
@@ -547,8 +560,15 @@ namespace Ipopt
         c_eval_time_.Start();
         bool success = nlp_->Eval_c(*unscaled_x, *unscaled_c);
         c_eval_time_.End();
-        ASSERT_EXCEPTION(success && IsFiniteNumber(unscaled_c->Nrm2()),
-                         Eval_Error, "Error evaluating the equality constraints");
+        if (!success || !IsFiniteNumber(unscaled_c->Nrm2())) {
+          if (check_derivatives_for_naninf_ && !IsFiniteNumber(unscaled_c->Nrm2())) {
+            jnlst_->Printf(J_WARNING, J_NLP,
+                           "The equality constraints contain an invalid number\n");
+            unscaled_c->Print(*jnlst_, J_MOREDETAILED, J_MAIN, "unscaled_c");
+            jnlst_->FlushBuffer();
+          }
+          THROW_EXCEPTION(Eval_Error, "Error evaluating the equality constraints");
+        }
         retValue = NLP_scaling()->apply_vector_scaling_c(ConstPtr(unscaled_c));
         c_cache_.AddCachedResult1Dep(retValue, x);
       }
@@ -582,8 +602,15 @@ namespace Ipopt
         bool success = nlp_->Eval_d(*unscaled_x, *unscaled_d);
         d_eval_time_.End();
         DBG_PRINT_VECTOR(2, "unscaled_d", *unscaled_d);
-        ASSERT_EXCEPTION(success && IsFiniteNumber(unscaled_d->Nrm2()),
-                         Eval_Error, "Error evaluating the inequality constraints");
+        if (!success || !IsFiniteNumber(unscaled_d->Nrm2())) {
+          if (check_derivatives_for_naninf_ && !IsFiniteNumber(unscaled_d->Nrm2())) {
+            jnlst_->Printf(J_WARNING, J_NLP,
+                           "The inequality constraints contain an invalid number\n");
+            unscaled_d->Print(*jnlst_, J_MOREDETAILED, J_MAIN, "unscaled_d");
+            jnlst_->FlushBuffer();
+          }
+          THROW_EXCEPTION(Eval_Error, "Error evaluating the inequality constraints");
+        }
         retValue = NLP_scaling()->apply_vector_scaling_d(ConstPtr(unscaled_d));
         d_cache_.AddCachedResult1Dep(retValue, x);
       }
@@ -623,6 +650,8 @@ namespace Ipopt
           if (!unscaled_jac_c->HasValidNumbers()) {
             jnlst_->Printf(J_WARNING, J_NLP,
                            "The Jacobian for the equality constraints contains an invalid number\n");
+            unscaled_jac_c->Print(*jnlst_, J_MOREDETAILED, J_MAIN, "unscaled_jac_c");
+            jnlst_->FlushBuffer();
             THROW_EXCEPTION(Eval_Error, "The Jacobian for the equality constraints contains an invalid number");
           }
         }
@@ -662,6 +691,16 @@ namespace Ipopt
         bool success = nlp_->Eval_jac_d(*unscaled_x, *unscaled_jac_d);
         jac_d_eval_time_.End();
         ASSERT_EXCEPTION(success, Eval_Error, "Error evaluating the jacobian of the inequality constraints");
+        if (check_derivatives_for_naninf_) {
+          if (!unscaled_jac_d->HasValidNumbers()) {
+            jnlst_->Printf(J_WARNING, J_NLP,
+                           "The Jacobian for the inequality constraints contains an invalid number\n");
+
+            unscaled_jac_d->Print(*jnlst_, J_MOREDETAILED, J_MAIN, "unscaled_jac_d");
+            jnlst_->FlushBuffer();
+            THROW_EXCEPTION(Eval_Error, "The Jacobian for the inequality constraints contains an invalid number");
+          }
+        }
         retValue = NLP_scaling()->apply_jac_d_scaling(ConstPtr(unscaled_jac_d));
         jac_d_cache_.AddCachedResult1Dep(retValue, GetRawPtr(dep));
       }
@@ -709,6 +748,16 @@ namespace Ipopt
       bool success = nlp_->Eval_h(*unscaled_x, scaled_obj_factor, *unscaled_yc, *unscaled_yd, *unscaled_h);
       h_eval_time_.End();
       ASSERT_EXCEPTION(success, Eval_Error, "Error evaluating the hessian of the lagrangian");
+      if (check_derivatives_for_naninf_) {
+        if (!unscaled_h->HasValidNumbers()) {
+          jnlst_->Printf(J_WARNING, J_NLP,
+                         "The Lagrangian Hessian contains an invalid number\n");
+
+          unscaled_h->Print(*jnlst_, J_MOREDETAILED, J_MAIN, "unscaled_h");
+          jnlst_->FlushBuffer();
+          THROW_EXCEPTION(Eval_Error, "The Lagrangian Hessian contains an invalid number");
+        }
+      }
       retValue = NLP_scaling()->apply_hessian_scaling(ConstPtr(unscaled_h));
       h_cache_.AddCachedResult(retValue, deps, scalar_deps);
     }
@@ -894,37 +943,93 @@ namespace Ipopt
       return;
 
     jnlst.Printf(level, category,
-                 "Function Evaluations................: %10.3f\n",
-                 TotalFunctionEvaluationCPUTime());
+                 "Function Evaluations................: %10.3f (sys: %10.3f wall: %10.3f)\n",
+                 TotalFunctionEvaluationCpuTime(),
+                 TotalFunctionEvaluationSysTime(),
+                 TotalFunctionEvaluationWallclockTime());
     jnlst.Printf(level, category,
-                 " Objective function.................: %10.3f\n",
-                 f_eval_time_.TotalTime());
+                 " Objective function.................: %10.3f (sys: %10.3f wall: %10.3f)\n",
+                 f_eval_time_.TotalCpuTime(),
+                 f_eval_time_.TotalSysTime(),
+                 f_eval_time_.TotalWallclockTime());
     jnlst.Printf(level, category,
-                 " Equality constraints...............: %10.3f\n",
-                 c_eval_time_.TotalTime());
+                 " Objective function gradient........: %10.3f (sys: %10.3f wall: %10.3f)\n",
+                 grad_f_eval_time_.TotalCpuTime(),
+                 grad_f_eval_time_.TotalSysTime(),
+                 grad_f_eval_time_.TotalWallclockTime());
     jnlst.Printf(level, category,
-                 " Inequality constraints.............: %10.3f\n",
-                 d_eval_time_.TotalTime());
+                 " Equality constraints...............: %10.3f (sys: %10.3f wall: %10.3f)\n",
+                 c_eval_time_.TotalCpuTime(),
+                 c_eval_time_.TotalSysTime(),
+                 c_eval_time_.TotalWallclockTime());
     jnlst.Printf(level, category,
-                 " Equality constraint Jacobian.......: %10.3f\n",
-                 jac_c_eval_time_.TotalTime());
+                 " Inequality constraints.............: %10.3f (sys: %10.3f wall: %10.3f)\n",
+                 d_eval_time_.TotalCpuTime(),
+                 d_eval_time_.TotalSysTime(),
+                 d_eval_time_.TotalWallclockTime());
     jnlst.Printf(level, category,
-                 " Inequality constraint Jacobian.....: %10.3f\n",
-                 jac_d_eval_time_.TotalTime());
+                 " Equality constraint Jacobian.......: %10.3f (sys: %10.3f wall: %10.3f)\n",
+                 jac_c_eval_time_.TotalCpuTime(),
+                 jac_c_eval_time_.TotalSysTime(),
+                 jac_c_eval_time_.TotalWallclockTime());
     jnlst.Printf(level, category,
-                 " Lagrangian Hessian.................: %10.3f\n",
-                 h_eval_time_.TotalTime());
+                 " Inequality constraint Jacobian.....: %10.3f (sys: %10.3f wall: %10.3f)\n",
+                 jac_d_eval_time_.TotalCpuTime(),
+                 jac_d_eval_time_.TotalSysTime(),
+                 jac_d_eval_time_.TotalWallclockTime());
+    jnlst.Printf(level, category,
+                 " Lagrangian Hessian.................: %10.3f (sys: %10.3f wall: %10.3f)\n",
+                 h_eval_time_.TotalCpuTime(),
+                 h_eval_time_.TotalSysTime(),
+                 h_eval_time_.TotalWallclockTime());
   }
 
   Number
-  OrigIpoptNLP::TotalFunctionEvaluationCPUTime() const
+  OrigIpoptNLP::TotalFunctionEvaluationCpuTime() const
   {
-    return f_eval_time_.TotalTime()+
-           c_eval_time_.TotalTime()+
-           d_eval_time_.TotalTime()+
-           jac_c_eval_time_.TotalTime()+
-           jac_d_eval_time_.TotalTime()+
-           h_eval_time_.TotalTime();
+    return f_eval_time_.TotalCpuTime()+
+           grad_f_eval_time_.TotalCpuTime()+
+           c_eval_time_.TotalCpuTime()+
+           d_eval_time_.TotalCpuTime()+
+           jac_c_eval_time_.TotalCpuTime()+
+           jac_d_eval_time_.TotalCpuTime()+
+           h_eval_time_.TotalCpuTime();
+  }
+
+  Number
+  OrigIpoptNLP::TotalFunctionEvaluationSysTime() const
+  {
+    return f_eval_time_.TotalSysTime()+
+           grad_f_eval_time_.TotalSysTime()+
+           c_eval_time_.TotalSysTime()+
+           d_eval_time_.TotalSysTime()+
+           jac_c_eval_time_.TotalSysTime()+
+           jac_d_eval_time_.TotalSysTime()+
+           h_eval_time_.TotalSysTime();
+  }
+
+  Number
+  OrigIpoptNLP::TotalFunctionEvaluationWallclockTime() const
+  {
+    return f_eval_time_.TotalWallclockTime()+
+           grad_f_eval_time_.TotalWallclockTime()+
+           c_eval_time_.TotalWallclockTime()+
+           d_eval_time_.TotalWallclockTime()+
+           jac_c_eval_time_.TotalWallclockTime()+
+           jac_d_eval_time_.TotalWallclockTime()+
+           h_eval_time_.TotalWallclockTime();
+  }
+
+  void
+  OrigIpoptNLP::ResetTimes()
+  {
+    f_eval_time_.Reset();
+    grad_f_eval_time_.Reset();
+    c_eval_time_.Reset();
+    d_eval_time_.Reset();
+    jac_c_eval_time_.Reset();
+    jac_d_eval_time_.Reset();
+    h_eval_time_.Reset();
   }
 
   SmartPtr<const Vector>
