@@ -1,16 +1,18 @@
-// Copyright (C) 2005, 2008 International Business Machines and others.
+// Copyright (C) 2005, 2010 International Business Machines and others.
 // All Rights Reserved.
 // This code is published under the Common Public License.
 //
-// $Id: IpPardisoSolverInterface.cpp 1390 2009-01-06 16:55:52Z andreasw $
+// $Id: IpPardisoSolverInterface.cpp 1792 2010-11-01 15:59:59Z andreasw $
 //
 // Authors:  Carl Laird, Andreas Waechter     IBM    2005-03-17
 //
 //           Olaf Schenk                      Univ of Basel 2005-09-20
 //                  - changed options, added PHASE_ flag
 
+
 #include "IpoptConfig.h"
 #include "IpPardisoSolverInterface.hpp"
+# include <math.h>
 
 #ifdef HAVE_CSTDIO
 # include <cstdio>
@@ -42,19 +44,44 @@
 # endif
 #endif
 
-/** Prototypes for Pardiso's subroutines */
+// determine the correct name of the Pardiso function
+#ifdef _MSC_VER
+# define PARDISOINIT_FUNC PARDISOINIT
+# define PARDISO_FUNC PARDISO
+#else
+# define PARDISOINIT_FUNC F77_FUNC(pardisoinit,PARDISOINIT)
+# define PARDISO_FUNC F77_FUNC(pardiso,PARDISO)
+#endif
+
+
+/* Prototypes for Pardiso's subroutines */
 extern "C"
 {
-  void F77_FUNC(pardisoinit,PARDISOINIT)(void* PT, const ipfint* MTYPE,
-                                         ipfint* IPARM);
-  void F77_FUNC(pardiso,PARDISO)(void** PT, const ipfint* MAXFCT,
-                                 const ipfint* MNUM, const ipfint* MTYPE,
-                                 const ipfint* PHASE, const ipfint* N,
-                                 const double* A, const ipfint* IA,
-                                 const ipfint* JA, const ipfint* PERM,
-                                 const ipfint* NRHS, ipfint* IPARM,
-                                 const ipfint* MSGLVL, double* B, double* X,
-                                 ipfint* ERROR);
+#ifndef HAVE_PARDISO_NEWINTERFACE
+  void PARDISOINIT_FUNC(void* PT, const ipfint* MTYPE, ipfint* IPARM);
+#else
+  // The following is a fix to allow linking with Pardiso library under Windows
+  void PARDISOINIT_FUNC(void* PT, const ipfint* MTYPE,
+                        const ipfint* SOLVER,
+                        ipfint* IPARM,
+                        double* DPARM,
+                        ipfint* ERROR);
+#endif
+  void PARDISO_FUNC(void** PT, const ipfint* MAXFCT,
+                    const ipfint* MNUM, const ipfint* MTYPE,
+                    const ipfint* PHASE, const ipfint* N,
+                    const double* A, const ipfint* IA,
+                    const ipfint* JA, const ipfint* PERM,
+                    const ipfint* NRHS, ipfint* IPARM,
+                    const ipfint* MSGLVL, double* B, double* X,
+                    ipfint* ERROR, double* DPARM);
+
+
+#ifdef PARDISO_MATCHING_PREPROCESS
+  void smat_reordering_pardiso_wsmp_(const ipfint* N, const ipfint* ia, const ipfint* ja, const double* a_, ipfint* a2, ipfint* ja2,  double* a2_,
+                                     ipfint* perm2,  double* scale2, ipfint* tmp2_, ipfint preprocess );
+#endif
+
 }
 
 namespace Ipopt
@@ -66,6 +93,15 @@ namespace Ipopt
   PardisoSolverInterface::PardisoSolverInterface()
       :
       a_(NULL),
+
+#ifdef PARDISO_MATCHING_PREPROCESS
+      ia2(NULL),
+      ja2(NULL),
+      a2_(NULL),
+      perm2(NULL),
+      scale2(NULL),
+#endif
+
       negevals_(-1),
       initialized_(false),
 
@@ -79,6 +115,7 @@ namespace Ipopt
 
     PT_ = new void*[64];
     IPARM_ = new ipfint[64];
+    DPARM_ = new double[64];
   }
 
   PardisoSolverInterface::~PardisoSolverInterface()
@@ -94,15 +131,25 @@ namespace Ipopt
       ipfint ERROR;
       ipfint idmy;
       double ddmy;
-      F77_FUNC(pardiso,PARDISO)(PT_, &MAXFCT_, &MNUM_, &MTYPE_, &PHASE, &N,
-                                &ddmy, &idmy, &idmy, &idmy, &NRHS, IPARM_,
-                                &MSGLVL_, &ddmy, &ddmy, &ERROR);
+      PARDISO_FUNC(PT_, &MAXFCT_, &MNUM_, &MTYPE_, &PHASE, &N,
+                   &ddmy, &idmy, &idmy, &idmy, &NRHS, IPARM_,
+                   &MSGLVL_, &ddmy, &ddmy, &ERROR, DPARM_);
       DBG_ASSERT(ERROR==0);
     }
 
     delete[] PT_;
     delete[] IPARM_;
+    delete[] DPARM_;
     delete[] a_;
+
+#ifdef PARDISO_MATCHING_PREPROCESS
+    delete[] ia2;
+    delete[] ja2;
+    delete[] a2_;
+    delete[] perm2;
+    delete[] scale2;
+#endif
+
   }
 
   void PardisoSolverInterface::RegisterOptions(SmartPtr<RegisteredOptions> roptions)
@@ -154,38 +201,58 @@ namespace Ipopt
       "Setting this option to \"yes\" essentially disables inertia check. "
       "This option makes the algorithm non-robust and easily fail, but it "
       "might give some insight into the necessity of inertia control.");
-    roptions->AddIntegerOption(
-      "pardiso_iter_tol_exponent",
-      "",
-      -14,
-      "");
-    roptions->AddIntegerOption(
-      "pardiso_dropping_schur_exponent",
-      "",
-      -3,
-      "");
-    roptions->AddIntegerOption(
-      "pardiso_dropping_factor_exponent",
-      "",
-      -3,
-      "");
-    roptions->AddIntegerOption(
-      "pardiso_inverse_norm_factor",
-      "",
-      500,
-      "");
-    roptions->AddIntegerOption(
+    roptions->AddLowerBoundedIntegerOption(
       "pardiso_max_iter",
+      "Maximum number of Krylov-Subspace Iteration",
+      1, 500,
+      "DPARM(1)");
+    roptions->AddBoundedNumberOption(
+      "pardiso_iter_relative_tol",
+      "Relative Residual Convergence",
+      0.0, true, 1.0, true, 1e-6,
+      "DPARM(2)");
+    roptions->AddLowerBoundedIntegerOption(
+      "pardiso_iter_coarse_size",
+      "Maximum Size of Coarse Grid Matrix",
+      1, 5000,
+      "DPARM(3)");
+    roptions->AddLowerBoundedIntegerOption(
+      "pardiso_iter_max_levels",
+      "Maximum Size of Grid Levels",
+      1, 10000,
+      "DPARM(4)");
+    roptions->AddBoundedNumberOption(
+      "pardiso_iter_dropping_factor",
+      "dropping value for incomplete factor",
+      0.0, true, 1.0, true, 0.5,
+      "DPARM(5)");
+    roptions->AddBoundedNumberOption(
+      "pardiso_iter_dropping_schur",
+      "dropping value for sparsify schur complement factor",
+      0.0, true, 1.0, true, 1e-1,
+      "DPARM(6)");
+    roptions->AddLowerBoundedIntegerOption(
+      "pardiso_iter_max_row_fill",
+      "max fill for each row",
+      1,10000000,
+      "DPARM(7)");
+    roptions->AddLowerBoundedNumberOption(
+      "pardiso_iter_inverse_norm_factor",
       "",
-      500,
-      "");
+      1, true, 5000000,
+      "DPARM(8)");
     roptions->AddStringOption2(
       "pardiso_iterative",
-      "",
+      "Switch on iterative solver in Pardiso library",
       "no",
       "no", "",
       "yes", "",
       "");
+    roptions->AddLowerBoundedIntegerOption(
+      "pardiso_max_droptol_corrections",
+      "Maximal number of decreases of drop tolerance during one solve.",
+      1, 4,
+      "This is relevant only for iterative Pardiso options.");
   }
 
   bool PardisoSolverInterface::InitializeImpl(const OptionsList& options,
@@ -205,24 +272,34 @@ namespace Ipopt
                             pardiso_out_of_core_power, prefix);
     options.GetBoolValue("pardiso_skip_inertia_check",
                          skip_inertia_check_, prefix);
-    bool pardiso_iterative;
-    options.GetBoolValue("pardiso_iterative", pardiso_iterative, prefix);
-    int pardiso_iter_tol_exponent;
-    options.GetIntegerValue("pardiso_iter_tol_exponent",
-                            pardiso_iter_tol_exponent, prefix);
-    int pardiso_dropping_schur_exponent;
-    options.GetIntegerValue("pardiso_dropping_schur_exponent",
-                            pardiso_dropping_schur_exponent, prefix);
-    int pardiso_dropping_factor_exponent;
-    options.GetIntegerValue("pardiso_dropping_factor_exponent",
-                            pardiso_dropping_factor_exponent, prefix);
-    int pardiso_inverse_norm_factor;
-    options.GetIntegerValue("pardiso_inverse_norm_factor",
-                            pardiso_inverse_norm_factor, prefix);
+    options.GetBoolValue("pardiso_iterative", pardiso_iterative_, prefix);
     int pardiso_max_iter;
     options.GetIntegerValue("pardiso_max_iter", pardiso_max_iter, prefix);
+    Number pardiso_iter_relative_tol;
+    options.GetNumericValue("pardiso_iter_relative_tol",
+                            pardiso_iter_relative_tol, prefix);
+    Index pardiso_iter_coarse_size;
+    options.GetIntegerValue("pardiso_iter_coarse_size",
+                            pardiso_iter_coarse_size, prefix);
+    Index pardiso_iter_max_levels;
+    options.GetIntegerValue("pardiso_iter_max_levels",
+                            pardiso_iter_max_levels, prefix);
+    Number pardiso_iter_dropping_factor;
+    options.GetNumericValue("pardiso_iter_dropping_factor",
+                            pardiso_iter_dropping_factor, prefix);
+    Number pardiso_iter_dropping_schur;
+    options.GetNumericValue("pardiso_iter_dropping_schur",
+                            pardiso_iter_dropping_schur, prefix);
+    Index pardiso_iter_max_row_fill;
+    options.GetIntegerValue("pardiso_iter_max_row_fill",
+                            pardiso_iter_max_row_fill, prefix);
+    Number pardiso_iter_inverse_norm_factor;
+    options.GetNumericValue("pardiso_iter_inverse_norm_factor",
+                            pardiso_iter_inverse_norm_factor, prefix);
     int pardiso_msglvl;
     options.GetIntegerValue("pardiso_msglvl", pardiso_msglvl, prefix);
+    options.GetIntegerValue("pardiso_max_droptol_corrections",
+                            pardiso_max_droptol_corrections_, prefix);
 
     // Number value = 0.0;
 
@@ -234,9 +311,9 @@ namespace Ipopt
       ipfint ERROR;
       ipfint idmy;
       double ddmy;
-      F77_FUNC(pardiso,PARDISO)(PT_, &MAXFCT_, &MNUM_, &MTYPE_, &PHASE, &N,
-                                &ddmy, &idmy, &idmy, &idmy, &NRHS, IPARM_,
-                                &MSGLVL_, &ddmy, &ddmy, &ERROR);
+      PARDISO_FUNC(PT_, &MAXFCT_, &MNUM_, &MTYPE_, &PHASE, &N,
+                   &ddmy, &idmy, &idmy, &idmy, &NRHS, IPARM_,
+                   &MSGLVL_, &ddmy, &ddmy, &ERROR, DPARM_);
       DBG_ASSERT(ERROR==0);
     }
 
@@ -248,9 +325,34 @@ namespace Ipopt
     delete[] a_;
     a_ = NULL;
 
+#ifdef PARDISO_MATCHING_PREPROCESS
+    delete[] ia2;
+    ia2 = NULL;
+
+    delete[] ja2;
+    ja2 = NULL;
+
+    delete[] a2_;
+    a2_ = NULL;
+
+    delete[] perm2;
+    perm2 = NULL;
+
+    delete[] scale2;
+    scale2 = NULL;
+#endif
+
     // Call Pardiso's initialization routine
     IPARM_[0] = 0;  // Tell it to fill IPARM with default values(?)
-    F77_FUNC(pardisoinit,PARDISOINIT)(PT_, &MTYPE_, IPARM_);
+
+#ifdef HAVE_PARDISO_NEWINTERFACE
+    ipfint ERROR = 0;
+    ipfint SOLVER = 0; // initialze only direct solver
+
+    PARDISOINIT_FUNC(PT_, &MTYPE_, &SOLVER, IPARM_, DPARM_, &ERROR);
+#else
+    PARDISOINIT_FUNC(PT_, &MTYPE_, IPARM_);
+#endif
 
     // Set some parameters for Pardiso
     IPARM_[0] = 1;  // Don't use the default values
@@ -303,21 +405,50 @@ namespace Ipopt
     IPARM_[29] = 1; // we need this for IPOPT interface
     IPARM_[29] = 1; // we need this for IPOPT interface
 
-    IPARM_[39] = 4 ;  // it was 4 max fill for factor
-    IPARM_[39] = 10 ;  // it was 4 max fill for factor
-    IPARM_[40] = 1 ;  // mantisse dropping value for schur complement
-    IPARM_[41] = pardiso_dropping_schur_exponent;
-    // it  exponent dropping value for schur complement
-    IPARM_[42] = pardiso_max_iter; // max number of iterations
-    IPARM_[43] = pardiso_inverse_norm_factor; // norm of the inverse for algebraic solver
-    IPARM_[44] = pardiso_dropping_factor_exponent ;  // exponent dropping value for incomplete factor
-    IPARM_[46] = 1 ;  // mantisse dropping value for incomplete factor
-    IPARM_[45] = pardiso_iter_tol_exponent ;  // residual tolerance
-    IPARM_[48] = pardiso_iterative ? 1 : 0 ;  // active direct solver
+    if (pardiso_iterative_) {
+#ifndef HAVE_PARDISO_NEWINTERFACE
+      THROW_EXCEPTION(OPTION_INVALID,
+                      "You chose to use the iterative version of Pardiso, but you need to use a Pardiso version of at least 4.0.");
+#endif
+      IPARM_[31] = 1 ;  // active direct solver
+
+      DPARM_[ 0] = pardiso_max_iter; // maximum number of Krylov-Subspace Iteration
+      // Default is 300
+      // 1 <= value <= e.g. 1000
+      DPARM_[ 1] = pardiso_iter_relative_tol; // Relative Residual Convergence
+      // e.g.  pardiso_iter_tol
+      // Default is 1e-6
+      // 1e-16 <= value < 1
+      DPARM_[ 2] = pardiso_iter_coarse_size; // Maximum Size of Coarse Grid Matrix
+      // e.g.  pardiso_coarse_grid
+      // Default is 5000
+      // 1 <= value < number of equations
+      DPARM_[ 3] = pardiso_iter_max_levels; // Maximum Number of Grid Levels
+      // e.g.  pardiso_max_grid
+      // Default is 10000
+      // 1 <= value < number of equations
+      DPARM_[ 4] = pardiso_iter_dropping_factor;  // dropping value for incomplete factor
+      // e.g.  pardiso_dropping_factor
+      // Default is 0.5
+      // 1e-16 <= value < 1
+      DPARM_[ 5] = pardiso_iter_dropping_schur;  // dropping value for sparsify schur complementfactor
+      // e.g.  pardiso_dropping_schur
+      // Default is 0.1
+      // 1e-16 <= value < 1
+      DPARM_[ 6] = pardiso_iter_max_row_fill;  // max fill for each row
+      // e.g.  pardiso_max_fill
+      // Default is 1000
+      // 1 <= value < 100000
+      DPARM_[ 7] = pardiso_iter_inverse_norm_factor;  // dropping value for sparsify schur complementfactor
+      // e.g.  pardiso_inverse_norm_factor
+      // Default is 500
+      // 2 <= value < 50000
+    }
+
     MSGLVL_ = pardiso_msglvl;
 
     // Option for the out of core variant
-    IPARM_[49] = pardiso_out_of_core_power;
+    //IPARM_[49] = pardiso_out_of_core_power;
 
     return true;
   }
@@ -424,7 +555,8 @@ namespace Ipopt
       else
         strcpy (mat_pref, "mat-ipopt");
 
-      sprintf (mat_name, "%s_%03d-%02d.iajaa", mat_pref, iter_cnt, sol_cnt);
+      Snprintf (mat_name, 127, "%s_%03d-%02d.iajaa",
+                mat_pref, iter_cnt, sol_cnt);
 
       // Open and write matrix file.
       mat_file = fopen (mat_name, "w");
@@ -446,6 +578,34 @@ namespace Ipopt
 
       fclose (mat_file);
     }
+    /* addtional matrix format */
+    if (getenv ("IPOPT_WRITE_MAT_MTX")) {
+      /* Write header */
+      FILE    *mat_file;
+      char     mat_name[128];
+      char     mat_pref[32];
+
+      ipfint   NNZ = ia[N]-1;
+      ipfint   i;
+      ipfint   j;
+
+      if (getenv ("IPOPT_WRITE_PREFIX"))
+        strcpy (mat_pref, getenv ("IPOPT_WRITE_PREFIX"));
+      else
+        strcpy (mat_pref, "mat-ipopt");
+
+      Snprintf (mat_name, 127, "%s_%03d-%02d.mtx",
+                mat_pref, iter_cnt, sol_cnt);
+
+      // Open and write matrix file.
+      mat_file = fopen (mat_name, "w");
+
+      for (i = 0; i < N; i++)
+        for (j = ia[i]; j < ia[i+1]-1; j++)
+          fprintf (mat_file, " %d %d %32.24e \n", i+1, ja[j-1], a_[j-1]);
+
+      fclose (mat_file);
+    }
   }
 
   ESymSolverStatus
@@ -476,12 +636,46 @@ namespace Ipopt
           IpData().TimingStats().LinearSystemSymbolicFactorization().Start();
         }
         PHASE = 11;
+
+#ifdef PARDISO_MATCHING_PREPROCESS
+        delete[] ia2;
+        ia2 = NULL;
+
+        delete[] ja2;
+        ja2 = NULL;
+
+        delete[] a2_;
+        a2_ = NULL;
+
+        delete[] perm2;
+        perm2 = NULL;
+
+        delete[] scale2;
+        scale2 = NULL;
+
+        ia2    = new ipfint[N+1];
+        ja2    = new ipfint[nonzeros_];
+        a2_    = new double[nonzeros_];
+        perm2  = new ipfint[N];
+        scale2 = new double[N];
+        ipfint* tmp2_  = new ipfint[N];
+
+        smat_reordering_pardiso_wsmp_(&N, ia, ja, a_, ia2, ja2, a2_, perm2, scale2, tmp2_, 0);
+
+        delete[] tmp2_;
+
+#endif
+
         Jnlst().Printf(J_DETAILED, J_LINEAR_ALGEBRA,
                        "Calling Pardiso for symbolic factorization.\n");
-        F77_FUNC(pardiso,PARDISO)(PT_, &MAXFCT_, &MNUM_, &MTYPE_,
-                                  &PHASE, &N, a_, ia, ja, &PERM,
-                                  &NRHS, IPARM_, &MSGLVL_, &B, &X,
-                                  &ERROR);
+        PARDISO_FUNC(PT_, &MAXFCT_, &MNUM_, &MTYPE_,
+#ifdef PARDISO_MATCHING_PREPROCESS
+                     &PHASE, &N, a2_, ia2, ja2, &PERM,
+#else
+                     &PHASE, &N, a_, ia, ja, &PERM,
+#endif
+                     &NRHS, IPARM_, &MSGLVL_, &B, &X,
+                     &ERROR, DPARM_);
         if (HaveIpData()) {
           IpData().TimingStats().LinearSystemSymbolicFactorization().End();
         }
@@ -525,150 +719,221 @@ namespace Ipopt
         debug_last_iter_ = 0;
       }
 
-      F77_FUNC(pardiso,PARDISO)(PT_, &MAXFCT_, &MNUM_, &MTYPE_,
-                                &PHASE, &N, a_, ia, ja, &PERM,
-                                &NRHS, IPARM_, &MSGLVL_, &B, &X,
-                                &ERROR);
-      if (HaveIpData()) {
-        IpData().TimingStats().LinearSystemFactorization().End();
-      }
+#ifdef PARDISO_MATCHING_PREPROCESS
+    {
+      ipfint* tmp2_  = new ipfint[N];
+      smat_reordering_pardiso_wsmp_ (&N, ia, ja, a_, ia2, ja2, a2_, perm2, scale2, tmp2_, 1);
+      delete[] tmp2_;
+    }
+#endif
 
-      if (ERROR==-7) {
-        Jnlst().Printf(J_MOREDETAILED, J_LINEAR_ALGEBRA,
-                       "Pardiso factorization returns ERROR = %d.  Matrix is singular.\n", ERROR);
-        return SYMSOLVER_SINGULAR;
-      }
-      else if (ERROR==-4) {
-        // I think this means that the matrix is singular
-        // OLAF said that this will never happen (ToDo)
-        return SYMSOLVER_SINGULAR;
-      }
-      else if (ERROR!=0 ) {
-        Jnlst().Printf(J_ERROR, J_LINEAR_ALGEBRA,
-                       "Error in Pardiso during factorization phase.  ERROR = %d.\n", ERROR);
-        return SYMSOLVER_FATAL_ERROR;
-      }
+    PARDISO_FUNC(PT_, &MAXFCT_, &MNUM_, &MTYPE_,
+#ifdef PARDISO_MATCHING_PREPROCESS
+                 &PHASE, &N, a2_, ia2, ja2, &PERM,
+#else
+                 &PHASE, &N, a_, ia, ja, &PERM,
+#endif
+                 &NRHS, IPARM_, &MSGLVL_, &B, &X,
+                 &ERROR, DPARM_);
+    if (HaveIpData()) {
+      IpData().TimingStats().LinearSystemFactorization().End();
+    }
 
-      negevals_ = Max(IPARM_[22], numberOfNegEVals);
-      if (IPARM_[13] != 0) {
-        Jnlst().Printf(J_DETAILED, J_LINEAR_ALGEBRA,
-                       "Number of perturbed pivots in factorization phase = %d.\n", IPARM_[13]);
-        if ( !pardiso_redo_symbolic_fact_only_if_inertia_wrong_ ||
-             (negevals_ != numberOfNegEVals) ) {
-          if (HaveIpData()) {
-            IpData().Append_info_string("Pn");
-          }
-          have_symbolic_factorization_ = false;
-          // We assume now that if there was just a symbolic
-          // factorization and we still have perturbed pivots, that
-          // the system is actually singular, if
-          // pardiso_repeated_perturbation_means_singular_ is true
-          if (just_performed_symbolic_factorization) {
-            if (pardiso_repeated_perturbation_means_singular_) {
-              if (HaveIpData()) {
-                IpData().Append_info_string("Ps");
-              }
-              return SYMSOLVER_SINGULAR;
-            }
-            else {
-              done = true;
+    if (ERROR==-7) {
+      Jnlst().Printf(J_MOREDETAILED, J_LINEAR_ALGEBRA,
+                     "Pardiso factorization returns ERROR = %d.  Matrix is singular.\n", ERROR);
+      return SYMSOLVER_SINGULAR;
+    }
+    else if (ERROR==-4) {
+      // I think this means that the matrix is singular
+      // OLAF said that this will never happen (ToDo)
+      return SYMSOLVER_SINGULAR;
+    }
+    else if (ERROR!=0 ) {
+      Jnlst().Printf(J_ERROR, J_LINEAR_ALGEBRA,
+                     "Error in Pardiso during factorization phase.  ERROR = %d.\n", ERROR);
+      return SYMSOLVER_FATAL_ERROR;
+    }
+
+    negevals_ = Max(IPARM_[22], numberOfNegEVals);
+    if (IPARM_[13] != 0) {
+      Jnlst().Printf(J_DETAILED, J_LINEAR_ALGEBRA,
+                     "Number of perturbed pivots in factorization phase = %d.\n", IPARM_[13]);
+      if ( !pardiso_redo_symbolic_fact_only_if_inertia_wrong_ ||
+           (negevals_ != numberOfNegEVals) ) {
+        if (HaveIpData()) {
+          IpData().Append_info_string("Pn");
+        }
+        have_symbolic_factorization_ = false;
+        // We assume now that if there was just a symbolic
+        // factorization and we still have perturbed pivots, that
+        // the system is actually singular, if
+        // pardiso_repeated_perturbation_means_singular_ is true
+        if (just_performed_symbolic_factorization) {
+          if (pardiso_repeated_perturbation_means_singular_) {
+            if (HaveIpData()) {
+              IpData().Append_info_string("Ps");
             }
+            return SYMSOLVER_SINGULAR;
           }
           else {
-            done = false;
+            done = true;
           }
         }
         else {
-          if (HaveIpData()) {
-            IpData().Append_info_string("Pp");
-          }
-          done = true;
+          done = false;
         }
       }
       else {
+        if (HaveIpData()) {
+          IpData().Append_info_string("Pp");
+        }
         done = true;
       }
     }
-
-    DBG_ASSERT(IPARM_[21]+IPARM_[22] == dim_);
-
-    // Check whether the number of negative eigenvalues matches the requested
-    // count
-    if (skip_inertia_check_) numberOfNegEVals=negevals_;
-
-    if (check_NegEVals && (numberOfNegEVals!=negevals_)) {
-      Jnlst().Printf(J_DETAILED, J_LINEAR_ALGEBRA,
-                     "Wrong inertia: required are %d, but we got %d.\n",
-                     numberOfNegEVals, negevals_);
-      return SYMSOLVER_WRONG_INERTIA;
+    else {
+      done = true;
     }
-
-    return SYMSOLVER_SUCCESS;
   }
 
-  ESymSolverStatus PardisoSolverInterface::Solve(const Index* ia,
-      const Index* ja,
-      Index nrhs,
-      double *rhs_vals)
-  {
-    DBG_START_METH("PardisoSolverInterface::Solve",dbg_verbosity);
+  DBG_ASSERT(IPARM_[21]+IPARM_[22] == dim_);
 
+  // Check whether the number of negative eigenvalues matches the requested
+  // count
+  if (skip_inertia_check_) numberOfNegEVals=negevals_;
+
+  if (check_NegEVals && (numberOfNegEVals!=negevals_)) {
+    Jnlst().Printf(J_DETAILED, J_LINEAR_ALGEBRA,
+                   "Wrong inertia: required are %d, but we got %d.\n",
+                   numberOfNegEVals, negevals_);
+    return SYMSOLVER_WRONG_INERTIA;
+  }
+
+  return SYMSOLVER_SUCCESS;
+}
+
+ESymSolverStatus PardisoSolverInterface::Solve(const Index* ia,
+    const Index* ja,
+    Index nrhs,
+    double *rhs_vals)
+{
+  DBG_START_METH("PardisoSolverInterface::Solve",dbg_verbosity);
+
+  if (HaveIpData()) {
+    IpData().TimingStats().LinearSystemBackSolve().Start();
+  }
+  // Call Pardiso to do the solve for the given right-hand sides
+  ipfint PHASE = 33;
+  ipfint N = dim_;
+  ipfint PERM;   // This should not be accessed by Pardiso
+  ipfint NRHS = nrhs;
+  double* X = new double[nrhs*dim_];
+
+  double* ORIG_RHS = new double[nrhs*dim_];
+  ipfint ERROR;
+  // Initialize solution with zero and save right hand side
+  for (int i = 0; i < N; i++) {
+    X[i] = 0.;
+    ORIG_RHS[i] = rhs_vals[i];
+  }
+
+  // Dump matrix to file if requested
+  Index iter_count = 0;
+  if (HaveIpData()) {
+    iter_count = IpData().iter_count();
+  }
+
+#ifdef PARDISO_MATCHING_PREPROCESS
+  write_iajaa_matrix (N, ia2, ja2, a2_, rhs_vals, iter_count, debug_cnt_);
+#else
+  write_iajaa_matrix (N,  ia,  ja,  a_, rhs_vals, iter_count, debug_cnt_);
+#endif
+
+  int attempts = 0;
+  const int max_attempts =
+    pardiso_iterative_ ? pardiso_max_droptol_corrections_+1: 1;
+
+  while (attempts < max_attempts) {
+
+
+#ifdef PARDISO_MATCHING_PREPROCESS
+    for (int i = 0; i < N; i++) {
+      rhs_vals[perm2[i]] = scale2[i] * ORIG_RHS[ i  ];
+    }
+    PARDISO_FUNC(PT_, &MAXFCT_, &MNUM_, &MTYPE_,
+                 &PHASE, &N, a2_, ia2, ja2, &PERM,
+                 &NRHS, IPARM_, &MSGLVL_, rhs_vals, X,
+                 &ERROR, DPARM_);
+    for (int i = 0; i < N; i++) {
+      X[i] = rhs_vals[ perm2[i]];
+    }
+    for (int i = 0; i < N; i++) {
+      rhs_vals[i] =  scale2[i]*X[i];
+    }
+
+#else
+    for (int i = 0; i < N; i++) {
+      rhs_vals[i] = ORIG_RHS[i];
+    }
+    PARDISO_FUNC(PT_, &MAXFCT_, &MNUM_, &MTYPE_,
+                 &PHASE, &N, a_, ia, ja, &PERM,
+                 &NRHS, IPARM_, &MSGLVL_, rhs_vals, X,
+                 &ERROR, DPARM_);
+#endif
+
+
+    if (ERROR <= -100 && ERROR >= -102) {
+      Jnlst().Printf(J_WARNING, J_LINEAR_ALGEBRA,
+                     "Iterative solver in Pardiso did not converge (ERROR = %d)\n", ERROR);
+      Jnlst().Printf(J_WARNING, J_LINEAR_ALGEBRA,
+                     "  Decreasing drop tolerances from DPARM_[41] = %e and DPARM_[44] = %e\n", DPARM_[41], DPARM_[44]);
+      PHASE = 23;
+      DPARM_[4] /= 2.0 ;
+      DPARM_[5] /= 2.0 ;
+      Jnlst().Printf(J_WARNING, J_LINEAR_ALGEBRA,
+                     "                               to DPARM_[41] = %e and DPARM_[44] = %e\n", DPARM_[41], DPARM_[44]);
+      attempts++;
+      ERROR = 0;
+    }
+    else {
+      attempts = max_attempts;
+    }
+  }
+
+  delete [] X;
+  delete [] ORIG_RHS;
+
+  if (IPARM_[6] != 0) {
+    Jnlst().Printf(J_DETAILED, J_LINEAR_ALGEBRA,
+                   "Number of iterative refinement steps = %d.\n", IPARM_[6]);
     if (HaveIpData()) {
-      IpData().TimingStats().LinearSystemBackSolve().Start();
+      IpData().Append_info_string("Pi");
     }
-    // Call Pardiso to do the solve for the given right-hand sides
-    ipfint PHASE = 33;
-    ipfint N = dim_;
-    ipfint PERM;   // This should not be accessed by Pardiso
-    ipfint NRHS = nrhs;
-    double* X = new double[nrhs*dim_];
-    ipfint ERROR;
-
-    // Dump matrix to file if requested
-    Index iter_count = 0;
-    if (HaveIpData()) {
-      iter_count = IpData().iter_count();
-    }
-    write_iajaa_matrix (N, ia, ja, a_, rhs_vals, iter_count, debug_cnt_);
-
-    F77_FUNC(pardiso,PARDISO)(PT_, &MAXFCT_, &MNUM_, &MTYPE_,
-                              &PHASE, &N, a_, ia, ja, &PERM,
-                              &NRHS, IPARM_, &MSGLVL_, rhs_vals, X,
-                              &ERROR);
-
-    delete [] X; /* OLAF/MICHAEL: do we really need X? */
-
-    if (IPARM_[6] != 0) {
-      Jnlst().Printf(J_DETAILED, J_LINEAR_ALGEBRA,
-                     "Number of iterative refinement steps = %d.\n", IPARM_[6]);
-      if (HaveIpData()) {
-        IpData().Append_info_string("Pi");
-      }
-    }
-
-    if (HaveIpData()) {
-      IpData().TimingStats().LinearSystemBackSolve().End();
-    }
-    if (ERROR!=0 ) {
-      Jnlst().Printf(J_ERROR, J_LINEAR_ALGEBRA,
-                     "Error in Pardiso during solve phase.  ERROR = %d.\n", ERROR);
-      return SYMSOLVER_FATAL_ERROR;
-    }
-    return SYMSOLVER_SUCCESS;
   }
 
-  Index PardisoSolverInterface::NumberOfNegEVals() const
-  {
-    DBG_START_METH("PardisoSolverInterface::NumberOfNegEVals",dbg_verbosity);
-    DBG_ASSERT(negevals_>=0);
-    return negevals_;
+  if (HaveIpData()) {
+    IpData().TimingStats().LinearSystemBackSolve().End();
   }
+  if (ERROR!=0 ) {
+    Jnlst().Printf(J_ERROR, J_LINEAR_ALGEBRA,
+                   "Error in Pardiso during solve phase.  ERROR = %d.\n", ERROR);
+    return SYMSOLVER_FATAL_ERROR;
+  }
+  return SYMSOLVER_SUCCESS;
+}
 
-  bool PardisoSolverInterface::IncreaseQuality()
-  {
-    // At the moment, I don't see how we could tell Pardiso to do better
-    // (maybe switch from IPARM[20]=1 to IPARM[20]=2?)
-    return false;
-  }
+Index PardisoSolverInterface::NumberOfNegEVals() const
+{
+  DBG_START_METH("PardisoSolverInterface::NumberOfNegEVals",dbg_verbosity);
+  DBG_ASSERT(negevals_>=0);
+  return negevals_;
+}
+
+bool PardisoSolverInterface::IncreaseQuality()
+{
+  // At the moment, I don't see how we could tell Pardiso to do better
+  // (maybe switch from IPARM[20]=1 to IPARM[20]=2?)
+  return false;
+}
 
 } // namespace Ipopt
