@@ -1,8 +1,8 @@
-// Copyright (C) 2004, 2008 International Business Machines and others.
+// Copyright (C) 2004, 2009 International Business Machines and others.
 // All Rights Reserved.
 // This code is published under the Common Public License.
 //
-// $Id: AmplTNLP.cpp 1224 2008-04-25 23:01:02Z andreasw $
+// $Id: AmplTNLP.cpp 1552 2009-08-10 14:21:05Z andreasw $
 //
 // Authors:  Carl Laird, Andreas Waechter     IBM    2004-08-13
 
@@ -10,7 +10,7 @@
 #ifdef HAVE_CONFIG_H
 # include "config_ipopt.h"
 #else
-# define PACKAGE_STRING "Ipopt 3.4.0"
+# define PACKAGE_VERSION IPOPT_VERSION
 #endif
 
 #include "AmplTNLP.hpp"
@@ -297,6 +297,52 @@ namespace Ipopt
     return true;
   }
 
+  bool AmplTNLP::get_var_con_metadata(Index n,
+                                      StringMetaDataMapType& var_string_md,
+                                      IntegerMetaDataMapType& var_integer_md,
+                                      NumericMetaDataMapType& var_numeric_md,
+                                      Index m,
+                                      StringMetaDataMapType& con_string_md,
+                                      IntegerMetaDataMapType& con_integer_md,
+                                      NumericMetaDataMapType& con_numeric_md)
+  {
+    ASL_pfgh* asl = asl_;
+    DBG_ASSERT(asl_);
+
+    // pick up the variable and constraints names if available
+    Index rlen = maxrownamelen;
+    Index clen = maxcolnamelen;
+
+    if (rlen > 0) {
+      std::vector<std::string> var_names(n);
+      for (Index i=0; i<n; i++) {
+        var_names[i] = var_name(i);
+      }
+      var_string_md_["idx_names"] = var_names;
+    }
+
+    if (clen > 0) {
+      std::vector<std::string> con_names(m);
+      for (Index i=0; i<m; i++) {
+        con_names[i] = con_name(i);
+      }
+      con_string_md_["idx_names"] = con_names;
+    }
+
+    if (var_string_md_.size() > 0 || var_integer_md_.size() > 0 || var_numeric_md_.size() > 0
+        || con_string_md_.size() > 0 || con_integer_md_.size() > 0 || con_numeric_md_.size() > 0) {
+      var_string_md = var_string_md_;
+      var_integer_md = var_integer_md_;
+      var_numeric_md = var_numeric_md_;
+      con_string_md = con_string_md_;
+      con_integer_md = con_integer_md_;
+      con_numeric_md = con_numeric_md_;
+      return true;
+    }
+
+    return false;
+  }
+
   bool AmplTNLP::get_bounds_info(Index n, Number* x_l, Number* x_u, Index m, Number* g_l, Number* g_u)
   {
     ASL_pfgh* asl = asl_;
@@ -587,6 +633,10 @@ namespace Ipopt
       message = "Maximum Number of Iterations Exceeded.";
       solve_result_num = 400;
     }
+    else if (status == CPUTIME_EXCEEDED) {
+      message = "Maximum CPU Time Exceeded.";
+      solve_result_num = 401;
+    }
     else if (status == STOP_AT_TINY_STEP) {
       message = "Search Direction becomes Too Small.";
       solve_result_num = 500;
@@ -623,7 +673,7 @@ namespace Ipopt
     }
 
     // Write the .sol file
-    message = " \n" PACKAGE_STRING ": " + message;
+    message = " \nIpopt " PACKAGE_VERSION ": " + message;
     write_solution_file(message.c_str());
   }
 
@@ -666,7 +716,6 @@ namespace Ipopt
       g = new double[m];
       allocated = true;
     }
-
     conval(const_cast<Number*>(x), g, (fint*)nerror_);
 
     if (allocated) {
@@ -1014,6 +1063,10 @@ namespace Ipopt
                                      "file_print_level",
                                      AmplOptionsList::Integer_Option,
                                      "Verbosity level for output file");
+    ampl_options_list->AddAmplOption("option_file_name",
+                                     "option_file_name",
+                                     AmplOptionsList::String_Option,
+                                     "File name of options file (default: ipopt.opt)");
 
     // Termination
     ampl_options_list->AddAmplOption("tol",
@@ -1028,6 +1081,10 @@ namespace Ipopt
                                      "max_iter",
                                      AmplOptionsList::Integer_Option,
                                      "Maximum number of iterations (same as max_iter)");
+    ampl_options_list->AddAmplOption("max_cpu_time",
+                                     "max_cpu_time",
+                                     AmplOptionsList::Number_Option,
+                                     "CPU time limit");
     ampl_options_list->AddAmplOption("compl_inf_tol",
                                      "compl_inf_tol",
                                      AmplOptionsList::Number_Option,
@@ -1271,7 +1328,7 @@ namespace Ipopt
                                              (void**)&nerror_);
 
     static const char sname_default[] = "ipopt";
-    static const char bsname_default[] = PACKAGE_STRING;
+    static const char bsname_default[] = "Ipopt " PACKAGE_VERSION;
     static const char opname_default[] = "ipopt_options";
     const char* sname;
     const char* bsname;
@@ -1431,6 +1488,21 @@ namespace Ipopt
     return dp->u.i;
   }
 
+  std::vector<Index>
+  AmplSuffixHandler::GetIntegerSuffixValues(Index n, std::string suffix_string,
+      Suffix_Source source) const
+  {
+    std::vector<Index> ret;
+    const Index* ptr = GetIntegerSuffixValues(suffix_string, source);
+    if (ptr) {
+      ret.reserve(n);
+      for (Index i=0; i<n; i++) {
+        ret.push_back(ptr[i]);
+      }
+    }
+    return ret;
+  }
+
   const Number*
   AmplSuffixHandler::GetNumberSuffixValues(std::string suffix_string,
       Suffix_Source source) const
@@ -1458,6 +1530,22 @@ namespace Ipopt
     SufDesc* dp = suf_get(suffix_string.c_str(), kind);
     return dp->u.r;
   }
+
+  std::vector<Number>
+  AmplSuffixHandler::GetNumberSuffixValues(Index n, std::string suffix_string,
+      Suffix_Source source) const
+  {
+    std::vector<Number> ret;
+    const Number* ptr = GetNumberSuffixValues(suffix_string, source);
+    if (ptr) {
+      ret.reserve(n);
+      for (Index i=0; i<n; i++) {
+        ret.push_back(ptr[i]);
+      }
+    }
+    return ret;
+  }
+
 } // namespace Ipopt
 
 
