@@ -2,7 +2,7 @@
 // All Rights Reserved.
 // This code is published under the Common Public License.
 //
-// $Id: IpOptionsList.cpp 1435 2009-04-24 17:09:05Z andreasw $
+// $Id: IpOptionsList.cpp 1624 2009-11-20 22:28:11Z andreasw $
 //
 // Authors:  Carl Laird, Andreas Waechter     IBM    2004-08-13
 
@@ -138,7 +138,7 @@ namespace Ipopt
                                     bool dont_print /* = false */)
   {
     char buffer[256];
-    sprintf(buffer, "%g", value);
+    Snprintf(buffer, 255, "%g", value);
 
     if (IsValid(reg_options_)) {
       SmartPtr<const RegisteredOption> option = reg_options_->GetOption(tag);
@@ -213,7 +213,7 @@ namespace Ipopt
                                     bool dont_print /* = false */)
   {
     char buffer[256];
-    sprintf(buffer, "%d", value);
+    Snprintf(buffer, 255, "%d", value);
 
     if (IsValid(reg_options_)) {
       SmartPtr<const RegisteredOption> option = reg_options_->GetOption(tag);
@@ -554,7 +554,7 @@ namespace Ipopt
   {
     lowercase_buffer_ = tag;
     for (Index i=0; i<(Index)tag.length(); i++) {
-      lowercase_buffer_[i] = tolower(tag[i]);
+      lowercase_buffer_[i] = (char)tolower(tag[i]);
     }
     return lowercase_buffer_;
   }
@@ -563,13 +563,13 @@ namespace Ipopt
   {
     list.erase();
     char buffer[256];
-    sprintf(buffer, "%40s   %-20s %s\n", "Name", "Value", "# times used");
+    Snprintf(buffer, 255, "%40s   %-20s %s\n", "Name", "Value", "# times used");
     list += buffer;
     for (std::map< std::string, OptionValue >::const_iterator p = options_.begin();
          p != options_.end();
          p++ ) {
-      sprintf(buffer, "%40s = %-20s %6d\n", p->first.c_str(),
-              p->second.Value().c_str(), p->second.Counter());
+      Snprintf(buffer, 255, "%40s = %-20s %6d\n", p->first.c_str(),
+               p->second.Value().c_str(), p->second.Counter());
       list += buffer;
     }
   }
@@ -578,7 +578,7 @@ namespace Ipopt
   {
     list.erase();
     char buffer[256];
-    sprintf(buffer, "%40s   %-20s %s\n", "Name", "Value", "used");
+    Snprintf(buffer, 255, "%40s   %-20s %s\n", "Name", "Value", "used");
     list += buffer;
     for (std::map< std::string, OptionValue >::const_iterator p = options_.begin();
          p != options_.end();
@@ -593,8 +593,8 @@ namespace Ipopt
         else {
           used = no;
         }
-        sprintf(buffer, "%40s = %-20s %4s\n", p->first.c_str(),
-                p->second.Value().c_str(), used);
+        Snprintf(buffer, 255, "%40s = %-20s %4s\n", p->first.c_str(),
+                 p->second.Value().c_str(), used);
         list += buffer;
       }
     }
@@ -757,7 +757,7 @@ namespace Ipopt
 
     // Now read the token
     while (!is.eof() && (inside_quotes || !isspace(c))) {
-      token += c;
+      token += (char)c;
       c = is.get();
       if (inside_quotes && (c=='"')) {
         inside_quotes = false;
