@@ -1,8 +1,8 @@
-// Copyright (C) 2004, 2008 International Business Machines and others.
+// Copyright (C) 2004, 2010 International Business Machines and others.
 // All Rights Reserved.
 // This code is published under the Common Public License.
 //
-// $Id: IpIpoptApplication.cpp 1221 2008-04-25 22:21:41Z andreasw $
+// $Id: IpIpoptApplication.cpp 1690 2010-06-21 17:58:45Z andreasw $
 //
 // Authors:  Carl Laird, Andreas Waechter     IBM    2004-09-02
 
@@ -19,6 +19,12 @@
 #include "IpInterfacesRegOp.hpp"
 #include "IpAlgorithmRegOp.hpp"
 #include "IpCGPenaltyRegOp.hpp"
+#include "IpNLPBoundsRemover.hpp"
+
+#ifdef BUILD_INEXACT
+# include "IpInexactRegOp.hpp"
+# include "IpInexactAlgBuilder.hpp"
+#endif
 
 #ifdef HAVE_CMATH
 # include <cmath>
@@ -32,6 +38,13 @@
 
 #include <fstream>
 
+// Factory to facilitate creating IpoptApplication objects from within a DLL
+
+Ipopt::IpoptApplication * IpoptApplicationFactory()
+{
+  return new Ipopt::IpoptApplication;
+}
+
 namespace Ipopt
 {
 #if COIN_IPOPT_VERBOSITY > 0
@@ -40,6 +53,11 @@ namespace Ipopt
 
   IpoptApplication::IpoptApplication(bool create_console_out /* = true */,
                                      bool create_empty /* = false */)
+      :
+      read_params_dat_(true),
+      inexact_algorithm_(false),
+      replace_bounds_(false),
+      skip_finalize_solution_call_(false)
   {
     options_ = new OptionsList();
     if (create_empty)
@@ -91,9 +109,13 @@ namespace Ipopt
                                      SmartPtr<OptionsList> options,
                                      SmartPtr<Journalist> jnlst)
       :
+      read_params_dat_(true),
       jnlst_(jnlst),
       reg_options_(reg_options),
-      options_(options)
+      options_(options),
+      inexact_algorithm_(false),
+      replace_bounds_(false),
+      skip_finalize_solution_call_(false)
   {}
 
   SmartPtr<IpoptApplication> IpoptApplication::clone()
@@ -103,12 +125,27 @@ namespace Ipopt
     retval->reg_options_ = RegOptions();
     *retval->options_ = *Options();
 
+    retval->read_params_dat_ = read_params_dat_;
+    retval->inexact_algorithm_ = inexact_algorithm_;
+    retval->replace_bounds_ = replace_bounds_;
+    retval->skip_finalize_solution_call_ = skip_finalize_solution_call_;
+
     return retval;
   }
 
   ApplicationReturnStatus
   IpoptApplication::Initialize(std::string params_file /*= "ipopt.opt"*/)
   {
+    std::string option_file_name;
+    options_->GetStringValue("option_file_name", option_file_name, "");
+    if (option_file_name != "") {
+      if (params_file !="") {
+        jnlst_->Printf(J_SUMMARY, J_MAIN, "Using option file \"%s\".\n\n",
+                       option_file_name.c_str());
+      }
+      params_file = option_file_name;
+    }
+
     std::ifstream is;
     if (params_file != "") {
       try {
@@ -141,50 +178,64 @@ namespace Ipopt
         options_->ReadFromStream(*jnlst_, is);
       }
 
-      Index ivalue;
-      options_->GetIntegerValue("print_level", ivalue, "");
-      EJournalLevel print_level = (EJournalLevel)ivalue;
-      SmartPtr<Journal> stdout_jrnl = jnlst_->GetJournal("console");
-      if (IsValid(stdout_jrnl)) {
-        // Set printlevel for stdout
-        stdout_jrnl->SetAllPrintLevels(print_level);
-        stdout_jrnl->SetPrintLevel(J_DBG, J_NONE);
-      }
+      bool no_output;
+      options_->GetBoolValue("suppress_all_output", no_output, "");
 
-      bool option_set;
-
-#if COIN_IPOPT_VERBOSITY > 0
-      // Set printlevel for debug
-      option_set = options_->GetIntegerValue("debug_print_level",
-                                             ivalue, "");
-      EJournalLevel debug_print_level;
-      if (option_set) {
-        debug_print_level = (EJournalLevel)ivalue;
+      if (no_output) {
+        jnlst_->DeleteAllJournals();
       }
       else {
-        debug_print_level = print_level;
-      }
-      SmartPtr<Journal> debug_jrnl = jnlst_->GetJournal("Debug");
-      if (IsNull(debug_jrnl)) {
-        debug_jrnl = jnlst_->AddFileJournal("Debug", "debug.out", J_ITERSUMMARY);
-      }
-      debug_jrnl->SetAllPrintLevels(debug_print_level);
-      debug_jrnl->SetPrintLevel(J_DBG, J_ALL);
-#endif
+        Index ivalue;
+        options_->GetIntegerValue("print_level", ivalue, "");
+        EJournalLevel print_level = (EJournalLevel)ivalue;
+        SmartPtr<Journal> stdout_jrnl = jnlst_->GetJournal("console");
+        if (IsValid(stdout_jrnl)) {
+          // Set printlevel for stdout
+          stdout_jrnl->SetAllPrintLevels(print_level);
+          stdout_jrnl->SetPrintLevel(J_DBG, J_NONE);
+        }
 
-      // Open an output file if required
-      std::string output_filename;
-      options_->GetStringValue("output_file", output_filename, "");
-      if (output_filename != "") {
-        EJournalLevel file_print_level;
-        option_set = options_->GetIntegerValue("file_print_level", ivalue, "");
+        bool option_set;
+
+#if COIN_IPOPT_VERBOSITY > 0
+        // Set printlevel for debug
+        option_set = options_->GetIntegerValue("debug_print_level",
+                                               ivalue, "");
+        EJournalLevel debug_print_level;
         if (option_set) {
-          file_print_level = (EJournalLevel)ivalue;
+          debug_print_level = (EJournalLevel)ivalue;
         }
         else {
-          file_print_level = print_level;
+          debug_print_level = print_level;
+        }
+        SmartPtr<Journal> debug_jrnl = jnlst_->GetJournal("Debug");
+        if (IsNull(debug_jrnl)) {
+          debug_jrnl = jnlst_->AddFileJournal("Debug", "debug.out", J_ITERSUMMARY);
+        }
+        debug_jrnl->SetAllPrintLevels(debug_print_level);
+        debug_jrnl->SetPrintLevel(J_DBG, J_ALL);
+#endif
+
+        // Open an output file if required
+        std::string output_filename;
+        options_->GetStringValue("output_file", output_filename, "");
+        if (output_filename != "") {
+          EJournalLevel file_print_level;
+          option_set = options_->GetIntegerValue("file_print_level", ivalue, "");
+          if (option_set) {
+            file_print_level = (EJournalLevel)ivalue;
+          }
+          else {
+            file_print_level = print_level;
+          }
+          bool openend = OpenOutputFile(output_filename, file_print_level);
+          if (!openend) {
+            jnlst_->Printf(J_ERROR, J_INITIALIZATION,
+                           "Error opening output file \"%s\"\n",
+                           output_filename.c_str());
+            return Invalid_Option;
+          }
         }
-        OpenOutputFile(output_filename, file_print_level);
       }
 
       // output a description of all the options
@@ -202,23 +253,28 @@ namespace Ipopt
           options_to_print.push_back("print_options_documentation");
           options_to_print.push_back("output_file");
           options_to_print.push_back("file_print_level");
+          options_to_print.push_back("option_file_name");
 
           options_to_print.push_back("#Termination");
           options_to_print.push_back("tol");
           options_to_print.push_back("max_iter");
+          options_to_print.push_back("max_cpu_time");
           options_to_print.push_back("dual_inf_tol");
           options_to_print.push_back("constr_viol_tol");
           options_to_print.push_back("compl_inf_tol");
           options_to_print.push_back("acceptable_tol");
+          options_to_print.push_back("acceptable_iter");
           options_to_print.push_back("acceptable_constr_viol_tol");
           options_to_print.push_back("acceptable_dual_inf_tol");
           options_to_print.push_back("acceptable_compl_inf_tol");
+          options_to_print.push_back("acceptable_obj_change_tol");
           options_to_print.push_back("diverging_iterates_tol");
 
           options_to_print.push_back("#NLP Scaling");
           options_to_print.push_back("obj_scaling_factor");
           options_to_print.push_back("nlp_scaling_method");
           options_to_print.push_back("nlp_scaling_max_gradient");
+          options_to_print.push_back("nlp_scaling_min_value");
 
           options_to_print.push_back("#NLP");
           options_to_print.push_back("bound_relax_factor");
@@ -239,7 +295,6 @@ namespace Ipopt
           options_to_print.push_back("bound_mult_init_val");
           options_to_print.push_back("constr_mult_init_max");
           options_to_print.push_back("bound_mult_init_method");
-          options_to_print.push_back("bound_mult_init_val");
 
           options_to_print.push_back("#Barrier Parameter");
           options_to_print.push_back("mehrotra_algorithm");
@@ -251,12 +306,14 @@ namespace Ipopt
           options_to_print.push_back("mu_max_fact");
           options_to_print.push_back("mu_max");
           options_to_print.push_back("mu_min");
+          options_to_print.push_back("mu_target");
           options_to_print.push_back("barrier_tol_factor");
           options_to_print.push_back("mu_linear_decrease_factor");
           options_to_print.push_back("mu_superlinear_decrease_power");
 
           options_to_print.push_back("#Multiplier Updates");
           options_to_print.push_back("alpha_for_y");
+          options_to_print.push_back("alpha_for_y_tol");
           options_to_print.push_back("recalc_y");
           options_to_print.push_back("recalc_y_feas_tol");
 
@@ -264,6 +321,7 @@ namespace Ipopt
           options_to_print.push_back("max_soc");
           options_to_print.push_back("watchdog_shortened_iter_trigger");
           options_to_print.push_back("watchdog_trial_iter_max");
+          options_to_print.push_back("accept_every_trial_step");
           options_to_print.push_back("corrector_type");
 
           options_to_print.push_back("#Warm Start");
@@ -278,6 +336,7 @@ namespace Ipopt
           options_to_print.push_back("#Restoration Phase");
           options_to_print.push_back("expect_infeasible_problem");
           options_to_print.push_back("expect_infeasible_problem_ctol");
+          options_to_print.push_back("expect_infeasible_problem_ytol");
           options_to_print.push_back("start_with_resto");
           options_to_print.push_back("soft_resto_pderror_reduction_factor");
           options_to_print.push_back("required_infeasibility_reduction");
@@ -330,6 +389,25 @@ namespace Ipopt
           options_to_print.push_back("ma57_pivtol");
           options_to_print.push_back("ma57_pivtolmax");
           options_to_print.push_back("ma57_pre_alloc");
+          options_to_print.push_back("ma57_pivot_order");
+          options_to_print.push_back("ma57_automatic_scaling");
+          options_to_print.push_back("ma57_block_size");
+          options_to_print.push_back("ma57_node_amalgamation");
+          options_to_print.push_back("ma57_small_pivot_flag");
+#endif
+
+#if defined(HAVE_MA77)
+
+          options_to_print.push_back("#MA77 Linear Solver");
+          options_to_print.push_back("ma77_print_level");
+          options_to_print.push_back("ma77_buffer_lpage");
+          options_to_print.push_back("ma77_buffer_npage");
+          options_to_print.push_back("ma77_file_size");
+          options_to_print.push_back("ma77_maxstore");
+          options_to_print.push_back("ma77_nemin");
+          options_to_print.push_back("ma77_small");
+          options_to_print.push_back("ma77_static");
+          options_to_print.push_back("ma77_u");
 #endif
 
 #ifdef COIN_HAS_MUMPS
@@ -346,6 +424,7 @@ namespace Ipopt
 #if defined(HAVE_PARDISO) || defined(HAVE_LINEARSOLVERLOADER)
 
           options_to_print.push_back("#Pardiso Linear Solver");
+          options_to_print.push_back("pardiso_msglvl");
           options_to_print.push_back("pardiso_matching_strategy");
           options_to_print.push_back("pardiso_out_of_core_power");
 #endif
@@ -358,6 +437,7 @@ namespace Ipopt
           options_to_print.push_back("wsmp_pivtol");
           options_to_print.push_back("wsmp_pivtolmax");
           options_to_print.push_back("wsmp_scaling");
+          options_to_print.push_back("wsmp_singularity_threshold");
 #endif
 
           reg_options_->OutputLatexOptionDocumentation(*jnlst_, options_to_print);
@@ -397,6 +477,16 @@ namespace Ipopt
         }
       }
 
+#ifdef BUILD_INEXACT
+      // Check if we are to use the inexact linear solver option
+      options_->GetBoolValue("inexact_algorithm", inexact_algorithm_, "");
+      // Change the default flags for the inexact algorithm
+      if (inexact_algorithm_) {
+        AddInexactDefaultOptions(*options_);
+      }
+#endif
+
+      options_->GetBoolValue("replace_bounds", replace_bounds_, "");
     }
     catch (OPTION_INVALID& exc) {
       exc.ReportException(*jnlst_, J_ERROR);
@@ -430,7 +520,7 @@ namespace Ipopt
     roptions->AddBoundedIntegerOption(
       "print_level",
       "Output verbosity level.",
-      -2, J_LAST_LEVEL-1, J_ITERSUMMARY,
+      0, J_LAST_LEVEL-1, J_ITERSUMMARY,
       "Sets the default verbosity level for console output. The "
       "larger this value the more detailed is the output.");
 
@@ -493,21 +583,61 @@ namespace Ipopt
       "If selected, the program will print the CPU usage (user time) for "
       "selected tasks.");
 
+    roptions->AddStringOption1(
+      "option_file_name",
+      "File name of options file (to overwrite default).",
+      "",
+      "*", "Any acceptable standard file name",
+      "By default, the name of the Ipopt options file is \"ipopt.opt\" - or "
+      "something else if specified in the IpoptApplication::Initialize call. "
+      "If this option is set by SetStringValue BEFORE the options file is "
+      "read, it specifies the name of the options file.  It does not make any "
+      "sense to specify this option within the options file.");
+
+    roptions->AddStringOption2(
+      "replace_bounds",
+      "Indicates if all variable bounds should be replaced by inequality constraints", "no",
+      "no", "leave bounds on variables",
+      "yes", "replace variable bounds by inequality constraints",
+      "This option must be set for the inexact algorithm");
+    roptions->AddStringOption2(
+      "skip_finalize_solution_call",
+      "Indicates if call to NLP::FinalizeSolution after optimization should be suppressed", "no",
+      "no", "call FinalizeSolution",
+      "yes", "do not call FinalizeSolution",
+      "In some Ipopt applications, the user might want to call the "
+      "FinalizeSolution method separately.  Setting this option to \"yes\" "
+      "will cause the IpoptApplication object to suppress the default call to "
+      "that method.");
+
     roptions->SetRegisteringCategory("Undocumented");
     roptions->AddStringOption2(
       "print_options_latex_mode",
       "Undocumented", "no",
       "no", "Undocumented",
       "yes", "Undocumented",
-      "Undocumented"
-    );
+      "Undocumented");
+    roptions->AddStringOption2(
+      "suppress_all_output",
+      "Undocumented", "no",
+      "no", "Undocumented",
+      "yes", "Undocumented",
+      "Undocumented");
+#ifdef BUILD_INEXACT
+    roptions->AddStringOption2(
+      "inexact_algorithm",
+      "Activate the version of Ipopt that allows iterative linear solvers.",
+      "no",
+      "no", "use default algorithm with direct linear solvers",
+      "yes", "use the EXPERIMENTAL iterative linear solver option",
+      "");
+#endif
   }
 
   ApplicationReturnStatus
   IpoptApplication::OptimizeTNLP(const SmartPtr<TNLP>& tnlp)
   {
     nlp_adapter_ = new TNLPAdapter(GetRawPtr(tnlp), ConstPtr(jnlst_));
-
     return OptimizeNLP(nlp_adapter_);
   }
 
@@ -541,10 +671,26 @@ namespace Ipopt
     try {
 
       if (IsNull(alg_builder)) {
-        alg_builder = new AlgorithmBuilder();
+#ifdef BUILD_INEXACT
+        if (inexact_algorithm_) {
+          alg_builder = new InexactAlgorithmBuilder();
+        }
+        else {
+#endif
+          alg_builder = new AlgorithmBuilder();
+#ifdef BUILD_INEXACT
+        }
+#endif
       }
 
-      alg_builder->BuildIpoptObjects(*jnlst_, *options_, "", nlp,
+      SmartPtr<NLP> use_nlp;
+      if (replace_bounds_) {
+        use_nlp = new NLPBoundsRemover(*nlp);
+      }
+      else {
+        use_nlp = nlp;
+      }
+      alg_builder->BuildIpoptObjects(*jnlst_, *options_, "", use_nlp,
                                      ip_nlp_, ip_data_, ip_cq_);
 
       alg_ = GetRawPtr(alg_builder->BuildBasicAlgorithm(*jnlst_, *options_, ""));
@@ -606,23 +752,26 @@ namespace Ipopt
     }
 
     statistics_ = NULL; /* delete old statistics */
+    // Get the pointers to the real objects (need to do it that
+    // awkwardly since otherwise we would have to include so many
+    // things in IpoptApplication, which a user would have to
+    // include, too (SmartPtr doesn't work otherwise on AIX)
+    IpoptAlgorithm* p2alg = static_cast<IpoptAlgorithm*> (GetRawPtr(alg_));
+    DBG_ASSERT(dynamic_cast<IpoptAlgorithm*> (GetRawPtr(alg_)));
+    IpoptData* p2ip_data = static_cast<IpoptData*> (GetRawPtr(ip_data_));
+    DBG_ASSERT(dynamic_cast<IpoptData*> (GetRawPtr(ip_data_)));
+    OrigIpoptNLP* p2ip_nlp = static_cast<OrigIpoptNLP*> (GetRawPtr(ip_nlp_));
+    DBG_ASSERT(dynamic_cast<OrigIpoptNLP*> (GetRawPtr(ip_nlp_)));
+    IpoptCalculatedQuantities* p2ip_cq = static_cast<IpoptCalculatedQuantities*> (GetRawPtr(ip_cq_));
+    DBG_ASSERT(dynamic_cast<IpoptCalculatedQuantities*> (GetRawPtr(ip_cq_)));
+
     // Reset Timing statistics
     ip_data_->TimingStats().ResetTimes();
+    p2ip_nlp->ResetTimes();
 
     ApplicationReturnStatus retValue = Internal_Error;
+    SolverReturn status = INTERNAL_ERROR;
     try {
-      // Get the pointers to the real objects (need to do it that
-      // awkwardly since otherwise we would have to include so many
-      // things in IpoptApplication, which a user would have to
-      // include, too (SmartPtr doesn't work otherwise on AIX)
-      IpoptAlgorithm* p2alg = static_cast<IpoptAlgorithm*> (GetRawPtr(alg_));
-      DBG_ASSERT(dynamic_cast<IpoptAlgorithm*> (GetRawPtr(alg_)));
-      IpoptData* p2ip_data = static_cast<IpoptData*> (GetRawPtr(ip_data_));
-      DBG_ASSERT(dynamic_cast<IpoptData*> (GetRawPtr(ip_data_)));
-      OrigIpoptNLP* p2ip_nlp = static_cast<OrigIpoptNLP*> (GetRawPtr(ip_nlp_));
-      DBG_ASSERT(dynamic_cast<OrigIpoptNLP*> (GetRawPtr(ip_nlp_)));
-      IpoptCalculatedQuantities* p2ip_cq = static_cast<IpoptCalculatedQuantities*> (GetRawPtr(ip_cq_));
-      DBG_ASSERT(dynamic_cast<IpoptCalculatedQuantities*> (GetRawPtr(ip_cq_)));
 
       // Set up the algorithm
       p2alg->Initialize(*jnlst_, *p2ip_nlp, *p2ip_data, *p2ip_cq,
@@ -650,7 +799,7 @@ namespace Ipopt
       }
 
       // Run the algorithm
-      SolverReturn status = p2alg->Optimize();
+      status = p2alg->Optimize();
 
       // Since all the output below doesn't make any sense in this
       // case, we rethrow the TOO_FEW_DOF exception here
@@ -721,14 +870,14 @@ namespace Ipopt
       jnlst_->Printf(J_SUMMARY, J_STATISTICS,
                      "Number of Lagrangian Hessian evaluations             = %d\n",
                      p2ip_nlp->h_evals());
-      Number time_overall_alg = p2ip_data->TimingStats().OverallAlgorithm().TotalTime();
-      Number time_funcs = p2ip_nlp->TotalFunctionEvaluationCPUTime();
+      Number cpu_time_overall_alg = p2ip_data->TimingStats().OverallAlgorithm().TotalCpuTime();
+      Number cpu_time_funcs = p2ip_nlp->TotalFunctionEvaluationCpuTime();
       jnlst_->Printf(J_SUMMARY, J_STATISTICS,
                      "Total CPU secs in IPOPT (w/o function evaluations)   = %10.3f\n",
-                     time_overall_alg-time_funcs);
+                     cpu_time_overall_alg-cpu_time_funcs);
       jnlst_->Printf(J_SUMMARY, J_STATISTICS,
                      "Total CPU secs in NLP function evaluations           = %10.3f\n",
-                     time_funcs);
+                     cpu_time_funcs);
 
       // Write timing statistics information
       if (print_timing_statistics) {
@@ -749,6 +898,10 @@ namespace Ipopt
         retValue = Maximum_Iterations_Exceeded;
         jnlst_->Printf(J_SUMMARY, J_MAIN, "\nEXIT: Maximum Number of Iterations Exceeded.\n");
       }
+      else if (status == CPUTIME_EXCEEDED) {
+        retValue = Maximum_CpuTime_Exceeded;
+        jnlst_->Printf(J_SUMMARY, J_MAIN, "\nEXIT: Maximum CPU time exceeded.\n");
+      }
       else if (status == STOP_AT_TINY_STEP) {
         retValue = Search_Direction_Becomes_Too_Small;
         jnlst_->Printf(J_SUMMARY, J_MAIN, "\nEXIT: Search Direction is becoming Too Small.\n");
@@ -790,16 +943,6 @@ namespace Ipopt
         jnlst_->Printf(J_SUMMARY, J_MAIN, "\nEXIT: INTERNAL ERROR: Unknown SolverReturn value - Notify IPOPT Authors.\n");
         return retValue;
       }
-      p2ip_nlp->FinalizeSolution(status,
-                                 *p2ip_data->curr()->x(),
-                                 *p2ip_data->curr()->z_L(),
-                                 *p2ip_data->curr()->z_U(),
-                                 *p2ip_cq->curr_c(), *p2ip_cq->curr_d(),
-                                 *p2ip_data->curr()->y_c(),
-                                 *p2ip_data->curr()->y_d(),
-                                 p2ip_cq->curr_f(),
-                                 p2ip_data,
-                                 p2ip_cq);
 
       if (status!=INVALID_NUMBER_DETECTED) {
         // Create a SolveStatistics object
@@ -810,21 +953,25 @@ namespace Ipopt
       exc.ReportException(*jnlst_, J_ERROR);
       jnlst_->Printf(J_SUMMARY, J_MAIN, "\nEXIT: Problem has too few degrees of freedom.\n");
       retValue = Not_Enough_Degrees_Of_Freedom;
+      status = TOO_FEW_DEGREES_OF_FREEDOM;
     }
     catch (OPTION_INVALID& exc) {
       exc.ReportException(*jnlst_, J_ERROR);
       jnlst_->Printf(J_SUMMARY, J_MAIN, "\nEXIT: Invalid option encountered.\n");
       retValue = Invalid_Option;
+      status = INVALID_OPTION;
     }
     catch (NO_FREE_VARIABLES_BUT_FEASIBLE& exc) {
       exc.ReportException(*jnlst_, J_MOREDETAILED);
       jnlst_->Printf(J_SUMMARY, J_MAIN, "\nEXIT: Optimal Solution Found.\n");
       retValue = Solve_Succeeded;
+      status = SUCCESS;
     }
     catch (NO_FREE_VARIABLES_AND_INFEASIBLE& exc) {
       exc.ReportException(*jnlst_, J_MOREDETAILED);
       jnlst_->Printf(J_SUMMARY, J_MAIN, "\nEXIT: Problem has only fixed variables and constraints are infeasible.\n");
       retValue = Infeasible_Problem_Detected;
+      status = LOCAL_INFEASIBILITY;
     }
     catch (IpoptException& exc) {
       exc.ReportException(*jnlst_, J_ERROR);
@@ -834,6 +981,7 @@ namespace Ipopt
     catch (std::bad_alloc) {
       retValue = Insufficient_Memory;
       jnlst_->Printf(J_SUMMARY, J_MAIN, "\nEXIT: Not enough memory.\n");
+      status = OUT_OF_MEMORY;
     }
     catch (...) {
       IpoptException exc("Unknown Exception caught in Ipopt", "Unknown File", -1);
@@ -841,6 +989,63 @@ namespace Ipopt
       retValue = NonIpopt_Exception_Thrown;
     }
 
+    if (IsValid(p2ip_data->curr()) && IsValid(p2ip_data->curr()->x())) {
+      SmartPtr<const Vector> c;
+      SmartPtr<const Vector> d;
+      SmartPtr<const Vector> zL;
+      SmartPtr<const Vector> zU;
+      SmartPtr<const Vector> yc;
+      SmartPtr<const Vector> yd;
+      Number obj = 0.;
+
+      switch (status) {
+      case SUCCESS:
+      case MAXITER_EXCEEDED:
+      case STOP_AT_TINY_STEP:
+      case STOP_AT_ACCEPTABLE_POINT:
+      case LOCAL_INFEASIBILITY:
+      case USER_REQUESTED_STOP:
+      case FEASIBLE_POINT_FOUND:
+      case DIVERGING_ITERATES:
+      case RESTORATION_FAILURE:
+      case ERROR_IN_STEP_COMPUTATION:
+        c = p2ip_cq->curr_c();
+        d = p2ip_cq->curr_d();
+        obj = p2ip_cq->curr_f();
+        zL = p2ip_data->curr()->z_L();
+        zU = p2ip_data->curr()->z_U();
+        yc = p2ip_data->curr()->y_c();
+        yd = p2ip_data->curr()->y_d();
+        break;
+      default: {
+          SmartPtr<Vector> tmp = p2ip_data->curr()->y_c()->MakeNew();
+          tmp->Set(0.);
+          c = ConstPtr(tmp);
+          yc = ConstPtr(tmp);
+          tmp = p2ip_data->curr()->y_d()->MakeNew();
+          tmp->Set(0.);
+          d = ConstPtr(tmp);
+          yd = ConstPtr(tmp);
+          tmp = p2ip_data->curr()->z_L()->MakeNew();
+          tmp->Set(0.);
+          zL = ConstPtr(tmp);
+          tmp = p2ip_data->curr()->z_U()->MakeNew();
+          tmp->Set(0.);
+          zU = ConstPtr(tmp);
+        }
+      }
+
+      options_->GetBoolValue("skip_finalize_solution_call",
+                             skip_finalize_solution_call_, "");
+
+      if (!skip_finalize_solution_call_) {
+        p2ip_nlp->FinalizeSolution(status,
+                                   *p2ip_data->curr()->x(),
+                                   *zL, *zU, *c, *d, *yc, *yd,
+                                   obj, p2ip_data, p2ip_cq);
+      }
+    }
+
     jnlst_->FlushBuffer();
 
     return retValue;
@@ -857,6 +1062,11 @@ namespace Ipopt
                                          print_level);
     }
 
+    // Check, if the output file could be created properly
+    if (IsNull(file_jrnl)) {
+      return false;
+    }
+
     file_jrnl->SetPrintLevel(J_DBG, J_NONE);
 
     return true;
@@ -868,6 +1078,9 @@ namespace Ipopt
     RegisterOptions_Algorithm(roptions);
     RegisterOptions_CGPenalty(roptions);
     RegisterOptions_LinearSolvers(roptions);
+#ifdef BUILD_INEXACT
+    RegisterOptions_Inexact(roptions);
+#endif
   }
 
   SmartPtr<SolveStatistics> IpoptApplication::Statistics()
@@ -875,4 +1088,29 @@ namespace Ipopt
     return statistics_;
   }
 
+  SmartPtr<IpoptNLP> IpoptApplication::IpoptNLPObject()
+  {
+    return ip_nlp_;
+  }
+
+  SmartPtr<IpoptData> IpoptApplication::IpoptDataObject()
+  {
+    return ip_data_;
+  }
+
+  SmartPtr<IpoptCalculatedQuantities> IpoptApplication::IpoptCQObject()
+  {
+    return ip_cq_;
+  }
+
+  SmartPtr<IpoptAlgorithm> IpoptApplication::AlgorithmObject()
+  {
+    return alg_;
+  }
+
+  void IpoptApplication::PrintCopyrightMessage()
+  {
+    IpoptAlgorithm::print_copyright_message(*jnlst_);
+  }
+
 } // namespace Ipopt
