@@ -1,3 +1,4 @@
+/* $Id: OSLindoSolver.cpp 3479 2010-04-26 07:14:52Z kmartin $ */
 /** @file LindoSolver.cpp
  * 
  * @author  Robert Fourer,  Jun Ma, Kipp Martin, 
@@ -20,6 +21,11 @@
 #include "OSFileUtil.h"
 #include "OSErrorClass.h"
 
+#include "OSDataStructures.h"
+#include "OSParameters.h" 
+#include "OSMathUtil.h"
+
+#include "CoinTime.hpp"
 
 
 #ifdef HAVE_CTIME
@@ -95,7 +101,8 @@ LindoSolver::LindoSolver():
 	m_msConName( NULL),
 	m_mcVarType( NULL),
 	m_mdObjConstant( 0),
-	osrlwriter( NULL)
+	osrlwriter( NULL),
+	cpuTime( 0)
 	
 {
 #ifdef DEBUG
@@ -111,6 +118,7 @@ LindoSolver::~LindoSolver() {
 	//delete[] m_mdVarLB ;
 	//delete[] m_mdVarUB ;
 	delete[] m_mmcVarName ;
+
 	m_mdConLB = NULL;
 	m_mdConUB = NULL;
 	m_msVarName = NULL;
@@ -157,7 +165,7 @@ void LindoSolver::buildSolverInstance() throw (ErrorClass) {
 			std::cout << "HERE I AM 2 !!!!!!!!!!!!!!!!!!!" << std::endl;
 		}		
 		//cout << osilwriter.writeOSiL( osinstance) << endl;
-		if(osinstance->getVariableNumber() <= 0)throw ErrorClass("Lindo requires decision variables");
+		if(osinstance->getVariableNumber() <  0)throw ErrorClass("Cannot have a negative number of decision variables");
 		std::cout << "Start process variables !!!!!!!!!" << std::endl;
 		if( !processVariables() ) throw ErrorClass("failed processing variables");
 		std::cout << "Finish process variables!!!!!!" << std::endl;
@@ -182,13 +190,31 @@ void LindoSolver::buildSolverInstance() throw (ErrorClass) {
 	}				
 }//end buildSolverInstance()
 
+
+
+void LindoSolver::setSolverOptions() throw (ErrorClass) {
+	try{
+		
+	}
+	catch(const ErrorClass& eclass){
+		std::cout << "THERE IS AN ERROR" << std::endl;
+		osresult->setGeneralMessage( eclass.errormsg);
+		osresult->setGeneralStatusType( "error");
+		osrl = osrlwriter->writeOSrL( osresult);
+		throw ErrorClass( osrl) ;
+	}				
+}//end setSolverOptions() 
+
  
 void LindoSolver::solve()  {
 	if( this->bCallbuildSolverInstance == false) buildSolverInstance();
 	try{
-
+		double start = CoinCpuTime();
 		if( optimize() != true) throw ErrorClass("problem optimizing model");
-
+		cpuTime = CoinCpuTime() - start;
+		osresult->setGeneralStatusType("normal");
+		osresult->setTime(cpuTime);
+		osrl = osrlwriter->writeOSrL( osresult);
 	}
 	catch(const ErrorClass& eclass){
 		osresult->setGeneralMessage( eclass.errormsg);
@@ -327,7 +353,7 @@ bool LindoSolver::addSlackVars(){
 bool LindoSolver::processVariables(){
 	int i;
 	try{
-		m_mcVarType = osinstance->getVariableTypes();
+		m_mcVarType = osinstance->getVariableTypes();	
 		m_mdVarLB = osinstance->getVariableLowerBounds();
 		m_mdVarUB  = osinstance->getVariableUpperBounds();
 		m_mmcVarName = new char*[ osinstance->getVariableNumber()];
@@ -337,6 +363,13 @@ bool LindoSolver::processVariables(){
 			//m_mmcVarName[i] = p;
 			m_mmcVarName[i] = &osinstance->getVariableNames()[  i][0];
 		}
+		//fix for Stefan
+		for(i = 0; i <  osinstance->getVariableNumber() ; i++){
+			if(	osinstance->getVariableTypes()[ i] == 'B'){
+				m_mdVarUB[ i] = 1.0;
+			}
+		}
+		
 		return true;
 	} 
 	catch(const ErrorClass& eclass){
@@ -438,7 +471,7 @@ bool LindoSolver::generateLindoModel(){
 		throw ;
 	}
 } // end generateLindoModel
-
+  
 
 bool LindoSolver::optimize(){
 	double *x,  *y, *z;
@@ -450,8 +483,11 @@ bool LindoSolver::optimize(){
 	int nSolStatus;
 	std::string description = "";	
 	// resultHeader infomration
-	if(osresult->setServiceName( "Solved using a LINDO service") != true)
-		throw ErrorClass("OSResult error: setServiceName");
+    if(osresult->setSolverInvoked( "LINDO Systems, Inc. Lindo API") != true)
+			throw ErrorClass("OSResult error: setSolverInvoked");
+    if(osresult->setServiceName( getVersionInfo()) != true)
+			throw ErrorClass("OSResult error: setServiceName");
+
 	if(osresult->setInstanceName(  osinstance->getInstanceName()) != true)
 		throw ErrorClass("OSResult error: setInstanceName");
 	//if(osresult->setJobID( osresultdata->jobID) != true)
@@ -503,7 +539,7 @@ bool LindoSolver::optimize(){
 		if(osresult->setSolutionNumber(  1) != true)
 			throw ErrorClass("OSResult error: setSolutionNumer");
 		cout << "Solution Status  = " <<  nSolStatus << endl;
-		osresult->setGeneralStatusType("success");
+		osresult->setGeneralStatusType("normal");
 		osresult->setSolutionStatus(solIdx, "optimal", description);	
 		x = new double[ osinstance->getVariableNumber() + m_iNumberNewSlacks];
 		srcost = new std::string[ osinstance->getVariableNumber() + m_iNumberNewSlacks];
@@ -534,7 +570,7 @@ bool LindoSolver::optimize(){
 					m_iLindoErrorCode = LSgetPrimalSolution( pModel_, x);  
 					lindoAPIErrorCheck("Error trying to obtain primal solution with NO integer variables present");
 				}
-				osresult->setPrimalVariableValues(solIdx, x);
+				osresult->setPrimalVariableValuesDense(solIdx, x);
 				// Get the dual values result
 				if( (osinstance->getNumberOfIntegerVariables() + osinstance->getNumberOfBinaryVariables() > 0)
 					|| (isNonlinear == false) ){
@@ -545,7 +581,7 @@ bool LindoSolver::optimize(){
 					m_iLindoErrorCode = LSgetDualSolution( pModel_, y);
 					lindoAPIErrorCheck("Error trying to obtain dual solution with NO integer variables present");
 				}
-				osresult->setDualVariableValues(solIdx, y);
+				osresult->setDualVariableValuesDense(solIdx, y);
 				// get the reduced cost result
 				if( ( osinstance->getNumberOfIntegerVariables() + osinstance->getNumberOfBinaryVariables() > 0)
 					|| (isNonlinear == false ) ) {
@@ -561,13 +597,13 @@ bool LindoSolver::optimize(){
 					int numberOfOtherVariableResult = 1;
 					int otherIdx = 0;
 					// first set the number of Other Variable Results
-					osresult->setNumberOfOtherVariableResult(solIdx, numberOfOtherVariableResult);
+					osresult->setNumberOfOtherVariableResults(solIdx, numberOfOtherVariableResult);
 					for(int i = 0; i <  osinstance->getVariableNumber() + m_iNumberNewSlacks; i++){
 						outStr << drcost[i];
 						srcost[ i] = outStr.str();
 						outStr.str("");
 					}
-					osresult->setAnOtherVariableResult(solIdx, otherIdx, "reduced costs", "the variable reduced costs", srcost);
+					osresult->setAnOtherVariableResultDense(solIdx, otherIdx, "reduced costs", "", "the variable reduced costs", srcost);
 					/* Get the value of the objective */
 					if( ( osinstance->getNumberOfIntegerVariables() + osinstance->getNumberOfBinaryVariables() > 0)
 						|| (isNonlinear == false ) ) {
@@ -578,7 +614,7 @@ bool LindoSolver::optimize(){
 						LSgetInfo( pModel_, LS_DINFO_GOP_OBJ, &z[0]);
 						lindoAPIErrorCheck("Error trying to obtain optimal objective value with NO integer variables present");	
 					}	
-					osresult->setObjectiveValues(solIdx, z);
+					osresult->setObjectiveValuesDense(solIdx, z);
 				}
 				break;
 			case 3:
@@ -590,7 +626,7 @@ bool LindoSolver::optimize(){
 			default:
 				osresult->setSolutionStatus(solIdx,  "other", description);
 		}
-		osrl = osrlwriter->writeOSrL( osresult);
+
 		delete[] x;
 		delete[] y;
 		delete[] z;
@@ -605,7 +641,7 @@ bool LindoSolver::optimize(){
 		osrl = osrlwriter->writeOSrL( osresult);
 		throw ;
 	}
-} //end solve
+} //end optimize
 
 
 bool LindoSolver::processQuadraticTerms(){
