@@ -2,7 +2,7 @@
 # define CPPAD_FORWARD_SWEEP_INCLUDED
 
 /* --------------------------------------------------------------------------
-CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-07 Bradley M. Bell
+CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-08 Bradley M. Bell
 
 CppAD is distributed under multiple licenses. This distribution is under
 the terms of the 
@@ -13,7 +13,7 @@ Please visit http://www.coin-or.org/CppAD/ for information on other licenses.
 -------------------------------------------------------------------------- */
 
 /*
-$begin ForwardSweep$$ $comment CppAD Developer Documentation$$
+$begin forward_sweep$$ $comment CppAD Developer Documentation$$
 $spell
 	Var
 	numvar
@@ -32,7 +32,7 @@ $spell
 $$
 
 $section Forward Computation of Taylor Coefficients$$
-$index ForwardSweep$$
+$index forward_sweep$$
 $index mode, forward$$
 $index forward, mode$$
 $index derivative, forward$$
@@ -40,11 +40,11 @@ $index Taylor coefficient, forward$$
 
 
 $head Syntax$$
-$syntax%size_t ForwardSweep(
+$syntax%size_t forward_sweep(
 	bool %print%,
 	size_t %d%,
 	size_t %numvar%,
-	TapeRec<%Base%> *%Rec%,
+	player<%Base%> *%Rec%,
 	size_t %J%,
 	Base *%Taylor%,
 )%$$
@@ -72,7 +72,7 @@ by the PripOp and PrivOp instructions.
 $head d$$
 Given the $th d-1$$ order Taylor coefficients matrix for all the variables,
 and the $th d$$ order Taylor coefficients for all the independent variables,
-$code ForwardSweep$$ computes the $th d$$ order Taylor coefficients 
+$code forward_sweep$$ computes the $th d$$ order Taylor coefficients 
 for all the other variables.
 
 
@@ -141,21 +141,26 @@ For $latex i = n+1, \ldots , numvar-1$$,
 $syntax%%Taylor%[%i% * %J% + %d%]%$$ is set equal to the
 $th d$$ order Taylor coefficient for the variable with index $italic i$$.
 
+$contents%
+        cppad/local/forward0sweep.hpp
+%$$
 
 $end
 ------------------------------------------------------------------------------
 */
+# include <cppad/local/forward0sweep.hpp>
+
 # define CPPAD_FORWARD_SWEEP_TRACE 0
 
 // BEGIN CppAD namespace
 namespace CppAD {
 
 template <class Base>
-size_t ForwardSweep(
+size_t forward_sweep(
 	bool                  print,
 	size_t                d,
 	size_t                numvar,
-	TapeRec<Base>        *Rec,
+	player<Base>         *Rec,
 	size_t                J,
 	Base                 *Taylor
 )
@@ -168,13 +173,13 @@ size_t ForwardSweep(
 	size_t        n_var;
 	size_t        n_ind;
 
-	const size_t   *ind;
+	const size_t   *ind = 0;
 	const Base       *P = 0;
 	const Base       *X = 0;
 	const Base       *Y = 0;
 
 	// used by CExp operator (left and right also used by Com operator)
-	const Base  *left, *right, *trueCase, *falseCase;
+	const Base  *left = 0, *right = 0, *trueCase = 0, *falseCase = 0;
 	const Base  zero = Base(0);
 
 	// used by Com operator
@@ -289,10 +294,9 @@ size_t ForwardSweep(
 			// -------------------------------------------------
 
 			case AcosOp:
+			// variables: acos(x),  sqrt(1 - x * x) 
 			CPPAD_ASSERT_UNKNOWN( n_ind == 1 );
 			CPPAD_ASSERT_UNKNOWN( ind[0] < i_var );
-
-			// acos(x) and sqrt(1 - x * x) are computed in pairs
 			CPPAD_ASSERT_UNKNOWN( n_var == 2);
 			CPPAD_ASSERT_UNKNOWN( (i_var+1) < numvar  );
 
@@ -304,10 +308,9 @@ size_t ForwardSweep(
 			// -------------------------------------------------
 
 			case AsinOp:
+			// variables: asin(x),  sqrt(1 - x * x) 
 			CPPAD_ASSERT_UNKNOWN( n_ind == 1 );
 			CPPAD_ASSERT_UNKNOWN( ind[0] < i_var );
-
-			// asin(x) and sqrt(1 - x * x) are computed in pairs
 			CPPAD_ASSERT_UNKNOWN( n_var == 2);
 			CPPAD_ASSERT_UNKNOWN( (i_var+1) < numvar  );
 
@@ -319,10 +322,9 @@ size_t ForwardSweep(
 			// -------------------------------------------------
 
 			case AtanOp:
+			// variables: atan(x),  1 + x * x 
 			CPPAD_ASSERT_UNKNOWN( n_ind == 1 );
 			CPPAD_ASSERT_UNKNOWN( ind[0] < i_var );
-
-			// atan(x) and 1 + x * x must be computed in pairs
 			CPPAD_ASSERT_UNKNOWN( n_var == 2);
 			CPPAD_ASSERT_UNKNOWN( (i_var+1) < numvar  );
 
@@ -422,10 +424,9 @@ size_t ForwardSweep(
 			// ---------------------------------------------------
 
 			case CosOp:
+			// variables: cos(x), sin(x)
 			CPPAD_ASSERT_UNKNOWN( n_ind == 1 );
 			CPPAD_ASSERT_UNKNOWN( ind[0] < i_var );
-
-			// cosine and sine must come in pairs
 			CPPAD_ASSERT_UNKNOWN( n_var == 2);
 			CPPAD_ASSERT_UNKNOWN( (i_var+1) < numvar  );
 
@@ -437,10 +438,9 @@ size_t ForwardSweep(
 			// ---------------------------------------------------
 
 			case CoshOp:
+			// variables: cosh(x), sinh(x)
 			CPPAD_ASSERT_UNKNOWN( n_ind == 1 );
 			CPPAD_ASSERT_UNKNOWN( ind[0] < i_var );
-
-			// hyperbolic cosine and sine must come in pairs
 			CPPAD_ASSERT_UNKNOWN( n_var == 2);
 			CPPAD_ASSERT_UNKNOWN( (i_var+1) < numvar  );
 
@@ -669,6 +669,7 @@ size_t ForwardSweep(
 			// -------------------------------------------------
 
 			case PowvpOp:
+			// variables: pow(x, y), log(x), y * log(x)
 			CPPAD_ASSERT_UNKNOWN( n_var == 3);
 			CPPAD_ASSERT_UNKNOWN( n_ind == 2 );
 			CPPAD_ASSERT_UNKNOWN( ind[0] < i_var);
@@ -693,6 +694,7 @@ size_t ForwardSweep(
 			// -------------------------------------------------
 
 			case PowpvOp:
+			// variables: pow(x, y), log(x), y * log(x)
 			CPPAD_ASSERT_UNKNOWN( n_var == 3);
 			CPPAD_ASSERT_UNKNOWN( n_ind == 2 );
 			CPPAD_ASSERT_UNKNOWN( ind[1] < i_var);
@@ -719,6 +721,7 @@ size_t ForwardSweep(
 			// -------------------------------------------------
 
 			case PowvvOp:
+			// variables: pow(x, y), log(x), y * log(x)
 			CPPAD_ASSERT_UNKNOWN( n_var == 3);
 			CPPAD_ASSERT_UNKNOWN( n_ind == 2 );
 			CPPAD_ASSERT_UNKNOWN( ind[0] < i_var);
@@ -755,7 +758,7 @@ size_t ForwardSweep(
 			// -------------------------------------------------
 
 			case PrivOp:
-			CPPAD_ASSERT_UNKNOWN( n_var == 1);
+			CPPAD_ASSERT_UNKNOWN( n_var == 0);
 			CPPAD_ASSERT_UNKNOWN( n_ind == 2 );
 			if( print & (d == 0) )
 			{	CPPAD_ASSERT_UNKNOWN( ind[0] < Rec->NumTxt() );
@@ -769,10 +772,9 @@ size_t ForwardSweep(
 			// -------------------------------------------------
 
 			case SinOp:
+			// variables: sin(x), cos(x)
 			CPPAD_ASSERT_UNKNOWN( n_ind == 1 );
 			CPPAD_ASSERT_UNKNOWN( ind[0] < i_var );
-
-			// sine and cosine must come in pairs
 			CPPAD_ASSERT_UNKNOWN( n_var == 2);
 			CPPAD_ASSERT_UNKNOWN( (i_var+1) < numvar  );
 
@@ -784,10 +786,9 @@ size_t ForwardSweep(
 			// -------------------------------------------------
 
 			case SinhOp:
+			// variables: sinh(x), cosh(x)
 			CPPAD_ASSERT_UNKNOWN( n_ind == 1 );
 			CPPAD_ASSERT_UNKNOWN( ind[0] < i_var );
-
-			// sine and cosine must come in pairs
 			CPPAD_ASSERT_UNKNOWN( n_var == 2);
 			CPPAD_ASSERT_UNKNOWN( (i_var+1) < numvar  );
 
