@@ -2,7 +2,7 @@
 # define CPPAD_REV_HES_SWEEP_INCLUDED
 
 /* --------------------------------------------------------------------------
-CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-07 Bradley M. Bell
+CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-08 Bradley M. Bell
 
 CppAD is distributed under multiple licenses. This distribution is under
 the terms of the 
@@ -37,7 +37,7 @@ $head Syntax$$
 $syntax%void RevHesSweep(
 	size_t %npv%,
 	size_t %numvar%,
-	const TapeRec<%Base%> *%Rec%,
+	const player<%Base%> *%Rec%,
 	size_t %TaylorColDim%,
 	const %Base% *%Taylor%,
 	const %Pack% *%ForJac%,
@@ -118,7 +118,7 @@ template <class Base, class Pack>
 void RevHesSweep(
 	size_t                npv,
 	size_t                numvar,
-	const TapeRec<Base>  *Rec,
+	const player<Base>   *Rec,
 	size_t                TaylorColDim,
 	const Base           *Taylor,
 	const Pack           *ForJac,
@@ -133,23 +133,23 @@ void RevHesSweep(
 	size_t        n_var;
 	size_t        n_ind;
 
-	const size_t   *ind;
+	const size_t   *ind = 0;
 
-	const Pack       *Xf;
-	Pack             *Xh;
+	const Pack       *Xf = 0;
+	Pack             *Xh = 0;
 
-	const Pack       *Yf;
-	Pack             *Yh;
+	const Pack       *Yf = 0;
+	Pack             *Yh = 0;
 
-	const Pack       *Zf;
-	const Pack       *Zr;
-	const Pack       *Zh;
+	const Pack       *Zf = 0;
+	const Pack       *Zr = 0;
+	const Pack       *Zh = 0;
 
 
 	// used by CExp operator 
-	const Base *left, *right;
-	Pack  *trueCaseh;
-	Pack  *falseCaseh;
+	const Base *left = 0, *right = 0;
+	Pack  *trueCaseh = 0;
+	Pack  *falseCaseh = 0;
 	Pack  zero(0);
 
 	size_t             j;
@@ -558,9 +558,7 @@ void RevHesSweep(
 			Yf = ForJac + ind[1] * npv;
 			Yh = RevHes + ind[1] * npv;
 			for(j = 0; j < npv; j++)
-			{	Xh[j] |= Zh[j] | (*Zr & Yf[j]); 
-				Yh[j] |= Zh[j] | (*Zr & Xf[j]); 
-			}
+				Yh[j] |= Zh[j] | (*Zr & Yf[j]); 
 			break;
 			// -------------------------------------------------
 
@@ -572,9 +570,7 @@ void RevHesSweep(
 			Xf = ForJac + ind[0] * npv;
 			Xh = RevHes + ind[0] * npv;
 			for(j = 0; j < npv; j++)
-			{	Xh[j] |= Zh[j] | (*Zr & Xf[j]); 
-				Yh[j] |= Zh[j] | (*Zr & Xf[j]); 
-			}
+				Xh[j] |= Zh[j] | (*Zr & Xf[j]); 
 			break;
 			// -------------------------------------------------
 
@@ -602,8 +598,7 @@ void RevHesSweep(
 			// -------------------------------------------------
 
 			case PrivOp:
-			// nvar should be zero for this case
-			CPPAD_ASSERT_UNKNOWN( n_var == 1);
+			CPPAD_ASSERT_UNKNOWN( n_var == 0);
 
 			break;
 			// -------------------------------------------------
